@@ -55,8 +55,6 @@ ConfigMgr* ConfigMgr::get()
 ConfigMgr::ConfigMgr()
 {
     addSubsys("scr");
-    addOption("scr", "subsys", "OGL",
-            "Whether to use OpenGL (OGL) or DirectFB (DFB) for video output");
     addOption("scr", "bpp", "24",
             "Screen bits per pixel. Valid values are 15, 16, 24 and 32.");
     addOption("scr", "fullscreen", "false",
@@ -66,21 +64,21 @@ ConfigMgr::ConfigMgr()
     addOption("scr", "windowheight", "0",
             "The height of the window to use. Contents are scaled.");
     addOption("scr", "usepow2textures", "false",
-            "OpenGL backend only: If set to true, use only power of 2 textures.");
+            "If set to true, use only power of 2 textures.");
     addOption("scr", "ycbcrmode", "shader",
-            "OpenGL backend only: How to render YCbCr surfaces. Valid values are"
+            "How to render YCbCr surfaces. Valid values are"
             " shader, mesa, apple and none.");
-    addOption("scr", "usergborder", "false",
-            "OpenGL backend only: Whether to use RGB (false) or BGR (true)"
-            " ordering internally for pixels.");
     addOption("scr", "usepixelbuffers", "true",
-            "OpenGL backend only: Whether to use pixel buffer objects.");
+            "Whether to use pixel buffer objects.");
     addOption("scr", "multisamplesamples", "1",
-            "OpenGL backend only: Whether to use multisampling and how many"
+            "Whether to use multisampling and how many"
             "samples per pixel to use.");
     addOption("scr", "gamma", "-1,-1,-1",
-            "OpenGL backend only: Display gamma correction values for red,"
+            "Display gamma correction values for red,"
             "green and blue.");
+    addOption("scr", "vsyncmode", "auto",
+            "How to synchronize the display refresh."
+            "Valid values are auto, ogl, dri and none.");
 
     m_sFName = "avgrc";
     loadFile("/etc/"+m_sFName);
@@ -149,7 +147,7 @@ bool ConfigMgr::getBoolOption(const std::string& sSubsys,
         return false;
     } else {
         AVG_TRACE(Logger::ERROR, 
-                "Unrecognized value for option "<<sName<<": " 
+                m_sFName << ": Unrecognized value for option "<<sName<<": " 
                 << *psOption << ". Must be true or false. Aborting.");
         exit(-1);
     }
@@ -158,6 +156,7 @@ bool ConfigMgr::getBoolOption(const std::string& sSubsys,
 int ConfigMgr::getIntOption(const std::string& sSubsys, 
         const std::string& sName, int Default) const
 {
+    errno = 0;
     const string * psOption = getOption(sSubsys, sName);
     if (psOption == 0) {
         return Default;
@@ -166,7 +165,7 @@ int ConfigMgr::getIntOption(const std::string& sSubsys,
     int rc = errno;
     if (rc == EINVAL || rc == ERANGE) {
         AVG_TRACE(Logger::ERROR,
-                "Unrecognized value for option "<<sName<<": " 
+                m_sFName << ": Unrecognized value for option "<<sName<<": " 
                 << *psOption << ". Must be an integer. Aborting.");
         exit(-1);
     }
@@ -183,7 +182,7 @@ void ConfigMgr::getGammaOption(const std::string& sSubsys,
     int rc = sscanf(psOption->c_str(), "%lf,%lf,%lf", Val, Val+1, Val+2);
     if (rc < 3) {
         AVG_TRACE(Logger::ERROR,
-                "Unrecognized value for option "<<sName<<": " 
+                m_sFName << ": Unrecognized value for option "<<sName<<": " 
                 << *psOption << ". Must be three comma-separated numbers. Aborting.");
         exit(-1);
     }
@@ -259,6 +258,7 @@ bool ConfigMgr::loadFile(const std::string& sPath) {
             }
             pSubsysNode = pSubsysNode->next;
         }
+        xmlFreeDoc(doc);
     } catch (Exception& e) {
         switch (e.GetCode()) {
             case AVG_ERR_OPTION_SUBSYS_UNKNOWN:
