@@ -22,34 +22,33 @@
 #ifndef _Profiler_H_ 
 #define _Profiler_H_
 
-#include "ProfilingZone.h"
+#include "ThreadProfiler.h"
 
-#include <list>
+#include <boost/thread.hpp>
+#include <boost/thread/mutex.hpp>
+
+#include <vector>
 
 namespace avg {
-    
+
 class Profiler {
 public:
     static Profiler& get();
     virtual ~Profiler();
  
-    void addZone(ProfilingZone& Zone);
-    void clear();
-    void start();
-    bool isRunning();
-    void setActiveZone(ProfilingZone * pZone);
     void dumpFrame();
     void dumpStatistics();
-    void reset();
 
+    // Interface to ThreadProfiler
+    void registerThreadProfiler(ThreadProfilerPtr pThreadProfiler);
+    ThreadProfilerPtr getThreadProfiler();
 
 private:
     Profiler();
 
-    typedef std::list<ProfilingZone*> ZoneList;
-    ZoneList m_Zones;
-    ProfilingZone * m_pActiveZone;
-    bool m_bRunning;
+    typedef std::vector<ThreadProfilerPtr> ThreadProfilerArray;
+    ThreadProfilerArray m_ThreadProfilers;
+    boost::mutex m_Mutex;
 };
 
 }
