@@ -22,6 +22,7 @@
 #include "Test.h"
 
 #include <iostream>
+#include <stdlib.h>
 
 using namespace std;
 
@@ -93,5 +94,16 @@ void Test::printResults()
         
 }
 
+std::string getSrcDir()
+{
+    char * pSrcDir = getenv("srcdir");
+    if (!pSrcDir) {
+        return "";
+    } else {
+        return string(pSrcDir)+"/";
+    }
+
+}
+
 }
 
