@@ -22,8 +22,6 @@
 #ifndef _Test_H_ 
 #define _Test_H_
 
-#include "CountedPointer.h"
-
 #include <boost/shared_ptr.hpp>
 
 #include <iostream>
@@ -60,12 +58,23 @@ private:
     std::string m_sName;
 };
 
+std::string getSrcDir();
+
 typedef boost::shared_ptr<Test> TestPtr;
 
+#define TEST_FAILED(s)                     \
+    cerr << string(m_IndentLevel+6, ' ') << s << endl;  \
+    test(false, __FILE__, __LINE__);
+
 #define TEST(b)                            \
     cerr << string(m_IndentLevel+4, ' ') << "  TEST(" << #b << ")" << endl;  \
     test(b, __FILE__, __LINE__);
     
+#define QUIET_TEST(b)                      \
+    if(!b) {                               \
+        cerr << string(m_IndentLevel+4, ' ') << "  TEST(" << #b << ")" << endl;  \
+    }                                      \
+    test(b, __FILE__, __LINE__);
 }
 #endif
 
