@@ -90,7 +90,11 @@ string getXmlChildrenAsString(const xmlDocPtr xmlDoc,
     s = (const char *)xmlBufferContent(pBuffer);
     int StartPos = s.find('>')+1;
     int EndPos = s.rfind('<')-1;
-    s = s.substr(StartPos, EndPos-StartPos+1);
+    if (StartPos > EndPos) {
+        s = "";
+    } else {
+        s = s.substr(StartPos, EndPos-StartPos+1);
+    }
     xmlBufferFree(pBuffer);
     return s;
 }
@@ -111,6 +115,14 @@ int getDefaultedIntAttr (const xmlNodePtr& xmlNode,
     return RVal;
 }
 
+int getRequiredIntAttr (const xmlNodePtr& xmlNode, 
+       const char * attr)
+{
+    int RVal;
+    xmlAttrToInt(xmlNode, attr, &RVal);
+    return RVal;
+}
+
 double getDefaultedDoubleAttr (const xmlNodePtr& xmlNode, 
        const char * attr, double def)
 {
@@ -144,8 +156,27 @@ string getRequiredStringAttr (const xmlNodePtr& xmlNode,
     return s;
 }
 
-static xmlExternalEntityLoader DefaultLoaderProc;
-static string g_sDTD;
+void writeMinMaxXMLNode(xmlTextWriterPtr writer, string sName, double Val[2])
+{
+    int rc;
+    rc = xmlTextWriterStartElement(writer, BAD_CAST sName.c_str());
+    writeAttribute(writer, "min", Val[0]);
+    writeAttribute(writer, "max", Val[1]);
+    rc = xmlTextWriterEndElement(writer);
+}
+
+void writePoint(xmlTextWriterPtr writer, string sName, const DPoint& Val)
+{
+    int rc;
+    rc = xmlTextWriterStartElement(writer, BAD_CAST sName.c_str());
+    writeAttribute(writer, "x", Val.x);
+    writeAttribute(writer, "y", Val.y);
+    rc = xmlTextWriterEndElement(writer);
+
+}
+
+static xmlExternalEntityLoader DefaultLoaderProc = 0;
+static std::map<string, string> g_DTDMap;
 
 xmlParserInputPtr
 DTDExternalEntityLoader(const char *URL, const char *ID,
@@ -153,18 +184,23 @@ DTDExternalEntityLoader(const char *URL, const char *ID,
 {
     xmlParserInputPtr ret;
     /* lookup for the fileID depending on ID */
-    if (!strcmp(URL, "avg.dtd")) {
-        ret = xmlNewStringInputStream(ctxt, (const xmlChar *)(g_sDTD.c_str()));
+    std::map<string, string>::iterator it = g_DTDMap.find(URL);
+
+    if (it != g_DTDMap.end()) {
+        ret = xmlNewStringInputStream(ctxt, (const xmlChar *)(it->second.c_str()));
+        return(ret);
+    } else {
+        ret = DefaultLoaderProc(URL, ID, ctxt);
         return(ret);
     }
-    ret = DefaultLoaderProc(URL, ID, ctxt);
-    return(ret);
 }
 
-void registerDTDEntityLoader(const char * pDTD)
+void registerDTDEntityLoader(const string& sID, const string& sDTD)
 {
-    g_sDTD = pDTD;
-    DefaultLoaderProc = xmlGetExternalEntityLoader();
+    g_DTDMap[sID] = sDTD;
+    if (!DefaultLoaderProc) {
+        DefaultLoaderProc = xmlGetExternalEntityLoader();
+    }
     xmlSetExternalEntityLoader(DTDExternalEntityLoader);
 }
 
