@@ -23,10 +23,12 @@
 #include "Pixel32.h"
 #include "Pixel24.h"
 #include "Pixel16.h"
+#include "Pixel8.h"
 #include "Filter3x3.h"
 
 #include "../base/Exception.h"
 #include "../base/Logger.h"
+#include "../base/ObjectCounter.h"
 
 #include <Magick++.h>
 #include <assert.h>
@@ -50,6 +52,7 @@ Bitmap::Bitmap(IntPoint Size, PixelFormat PF, const std::string& sName)
 {
 //    cerr << "Bitmap::Bitmap(" << Size << ", " << getPixelFormatString(m_PF) << ", " 
 //        << sName << ")" << endl;
+    ObjectCounter::get()->incRef(&typeid(*this));
     allocBits();
 }
 
@@ -59,6 +62,7 @@ Bitmap::Bitmap(IntPoint Size, PixelFormat PF, unsigned char * pBits,
       m_PF(PF),
       m_sName(sName)
 {
+    ObjectCounter::get()->incRef(&typeid(*this));
 //    cerr << "Bitmap::Bitmap(" << Size << ", " << getPixelFormatString(m_PF) << ", " 
 //        << (void *)pBits << ", " << Stride << ", " << bCopyBits << ", "
 //        << sName << ")" << endl;
@@ -72,6 +76,19 @@ Bitmap::Bitmap(const Bitmap& Orig)
       m_sName(Orig.getName()+" copy")
 {
 //    cerr << "Bitmap::Bitmap(Bitmap), Name: " << m_sName << endl;
+    ObjectCounter::get()->incRef(&typeid(*this));
+    initWithData(const_cast<unsigned char *>(Orig.getPixels()), Orig.getStride(), 
+            m_bOwnsBits);
+}
+
+Bitmap::Bitmap(const Bitmap& Orig, bool bOwnsBits)
+    : m_Size(Orig.getSize()),
+      m_PF(Orig.getPixelFormat()),
+      m_bOwnsBits(bOwnsBits),
+      m_sName(Orig.getName()+" copy")
+{
+//    cerr << "Bitmap::Bitmap(Bitmap), Name: " << m_sName << endl;
+    ObjectCounter::get()->incRef(&typeid(*this));
     initWithData(const_cast<unsigned char *>(Orig.getPixels()), Orig.getStride(), 
             m_bOwnsBits);
 }
@@ -84,6 +101,9 @@ Bitmap::Bitmap(Bitmap& Orig, const IntRect& Rect)
       m_bOwnsBits(false)
 {
 //    cerr << "Bitmap::Bitmap(Bitmap, " << Rect << "), Name: " << m_sName << endl;
+    ObjectCounter::get()->incRef(&typeid(*this));
+    assert(Rect.br.x <= Orig.getSize().x);
+    assert(Rect.br.y <= Orig.getSize().y);
     if (!Orig.getName().empty()) {
         m_sName = Orig.getName()+" part";
     } else {
@@ -122,11 +142,13 @@ Bitmap::Bitmap(const std::string& sURI)
         }
     }
     m_bOwnsBits = true;
+    ObjectCounter::get()->incRef(&typeid(*this));
 }
 
 Bitmap::~Bitmap()
 {
 //    cerr << "Bitmap::~Bitmap(), Name: " << m_sName << endl;
+    ObjectCounter::get()->decRef(&typeid(*this));
     if (m_bOwnsBits) {
         delete[] m_pBits;
     }
@@ -156,49 +178,87 @@ void Bitmap::copyPixels(const Bitmap & Orig)
     if (&Orig == this) {
         return;
     }
-    if (Orig.getPixelFormat() == YCbCr422 || Orig.getPixelFormat() == YCbCr411) {
-        if (m_PF == B8G8R8X8) {
-            YCbCrtoBGR(Orig);
-        } else {
-            Bitmap TempBmp(getSize(), B8G8R8X8, "TempColorConversion");
-            TempBmp.YCbCrtoBGR(Orig);
-            copyPixels(TempBmp);
+    if (Orig.getPixelFormat() == m_PF) {
+        const unsigned char * pSrc = Orig.getPixels();
+        unsigned char * pDest = m_pBits;
+        int Height = min(Orig.getSize().y, m_Size.y);
+        int LineLen = min(Orig.getLineLen(), getLineLen());
+        int SrcStride = Orig.getStride();
+        for (int y=0; y<Height; ++y) {
+            memcpy(pDest, pSrc, LineLen);
+            pDest += m_Stride;
+            pSrc += SrcStride;
         }
     } else {
-        if (Orig.getPixelFormat() == m_PF) {
-            const unsigned char * pSrc = Orig.getPixels();
-            unsigned char * pDest = m_pBits;
-            int Height = min(Orig.getSize().y, m_Size.y);
-            int LineLen = getLineLen();
-            for (int y=0; y<Height; ++y) {
-                memcpy(pDest, pSrc, LineLen);
-                pDest += m_Stride;
-                pSrc += Orig.getStride();
-            }
-        } else {
-            switch(m_PF) {
-                case B8G8R8A8:
-                case B8G8R8X8:
-                case A8B8G8R8:
-                case X8B8G8R8:
-                case R8G8B8A8:
-                case R8G8B8X8:
-                case A8R8G8B8:
-                case X8R8G8B8:
-                    createTrueColorCopy<Pixel32>(*this, Orig);
-                    break;
-                case B8G8R8:
-                case R8G8B8:
-                    createTrueColorCopy<Pixel24>(*this, Orig);
-                    break;
-                case B5G6R5:
-                case R5G6B5:
-                    createTrueColorCopy<Pixel16>(*this, Orig);
-                    break;
-                default:
-                    // Unimplemented conversion.
-                    assert(false);
-            }
+        switch (Orig.getPixelFormat()) {
+            case YCbCr422:
+            case YUYV422:
+            case YCbCr411:
+            case YCbCr420p:
+                if (m_PF == B8G8R8X8) {
+                    YCbCrtoBGR(Orig);
+                } else {
+                    Bitmap TempBmp(getSize(), B8G8R8X8, "TempColorConversion");
+                    TempBmp.YCbCrtoBGR(Orig);
+                    copyPixels(TempBmp);
+                }
+                break;
+            case I16:
+                if (m_PF == I8) {
+                    I16toI8(Orig);
+                } else {
+                    Bitmap TempBmp(getSize(), I8, "TempColorConversion");
+                    TempBmp.I16toI8(Orig);
+                    copyPixels(TempBmp);
+                }
+                break;
+            case I8:
+                switch(m_PF) {
+                    case I16:
+                        I8toI16(Orig);
+                        break;
+                    case B8G8R8X8:
+                    case B8G8R8A8:
+                    case R8G8B8X8:
+                    case R8G8B8A8:
+                    case B8G8R8:
+                    case R8G8B8:
+                        I8toRGB(Orig);
+                        break;
+                    default: 
+                        // Unimplemented conversion.
+                        assert(false);
+                }
+                break;
+            default:
+                switch(m_PF) {
+                    case B8G8R8A8:
+                    case B8G8R8X8:
+                    case A8B8G8R8:
+                    case X8B8G8R8:
+                    case R8G8B8A8:
+                    case R8G8B8X8:
+                    case A8R8G8B8:
+                    case X8R8G8B8:
+                        createTrueColorCopy<Pixel32>(*this, Orig);
+                        break;
+                    case B8G8R8:
+                    case R8G8B8:
+                        createTrueColorCopy<Pixel24>(*this, Orig);
+                        break;
+                    case B5G6R5:
+                    case R5G6B5:
+                        createTrueColorCopy<Pixel16>(*this, Orig);
+                        break;
+                    case I8:
+                        createTrueColorCopy<Pixel8>(*this, Orig);
+                        break;
+                    default:
+                        // Unimplemented conversion.
+                        cerr << "Can't convert " << Orig.getPixelFormatString() << " to " 
+                                << getPixelFormatString() << endl;
+                        assert(false);
+                }
         }
     }
 }
@@ -208,7 +268,7 @@ void Bitmap::save(const std::string& sFilename)
 //    cerr << "Bitmap::save()" << endl;
     string sPF;
     BitmapPtr pBmp;
-    // TODO: Not all of these are tested.
+    Magick::StorageType ChannelFormat = Magick::CharPixel;
     switch(m_PF) {
         case B5G6R5:
             pBmp = BitmapPtr(new Bitmap(m_Size, B8G8R8));
@@ -264,6 +324,11 @@ void Bitmap::save(const std::string& sFilename)
             pBmp->copyPixels(*this);
             sPF = "RGB";
             break;
+        case I16:   
+            pBmp = BitmapPtr(new Bitmap(*this));
+            ChannelFormat = Magick::ShortPixel;
+            sPF = "I";
+            break;
         case I8:
             pBmp = BitmapPtr(new Bitmap(*this));
             sPF = "I";
@@ -273,7 +338,7 @@ void Bitmap::save(const std::string& sFilename)
                     << endl;
             assert(false);
     }
-    Magick::Image Img(m_Size.x, m_Size.y, sPF, Magick::CharPixel, pBmp->getPixels());
+    Magick::Image Img(m_Size.x, m_Size.y, sPF, ChannelFormat, pBmp->getPixels());
     Img.write(sFilename);
 }
 
@@ -297,7 +362,7 @@ void Bitmap::setPixelFormat(PixelFormat PF)
     m_PF = PF;
 }
 
-std::string Bitmap::getPixelFormatString()
+std::string Bitmap::getPixelFormatString() const
 {
     return getPixelFormatString(m_PF);
 }
@@ -331,10 +396,14 @@ std::string Bitmap::getPixelFormatString(PixelFormat PF)
             return "X8R8G8B8";
         case I8:
             return "I8";
+        case I16:
+            return "I16";
         case YCbCr411:
             return "YCbCr411";
         case YCbCr422:
             return "YCbCr422";
+        case YUYV422:
+            return "YUYV422";
         case YCbCr420p:
             return "YCbCr420p";
         case YCbCrJ420p:
@@ -402,6 +471,7 @@ int Bitmap::getBytesPerPixel(PixelFormat PF)
             return 3;
         case B5G6R5:
         case R5G6B5:
+        case I16:
             return 2;
         case I8:
             return 1;
@@ -434,6 +504,22 @@ bool Bitmap::hasAlpha() const
             m_PF == A8R8G8B8);
 }
 
+HistogramPtr Bitmap::getHistogram(int Stride) const
+{
+    assert (m_PF == I8);
+    HistogramPtr pHist(new Histogram(256,0));
+    const unsigned char * pSrcLine = m_pBits;
+    for (int y=0; y < m_Size.y; y+=Stride) {
+        const unsigned char * pSrc = pSrcLine;
+        for (int x=0; x<m_Size.x; x+=Stride) {
+            (*pHist)[(*pSrc)]++;
+            pSrc+=Stride;
+        }
+        pSrcLine += m_Stride*Stride;
+    }
+    return pHist;
+}
+
 bool Bitmap::operator ==(const Bitmap & otherBmp)
 {
     // We allow Name, Stride and bOwnsBits to be different here, since we're looking for
@@ -588,7 +674,7 @@ void Bitmap::dump(bool bDumpPixels) const
 void Bitmap::initWithData(unsigned char * pBits, int Stride, bool bCopyBits)
 {
 //    cerr << "Bitmap::initWithData()" << endl;
-    if (m_PF == YCbCr422) {
+    if (m_PF == YCbCr422 || m_PF == YCbCr420p) {
         if (m_Size.x%2 == 1) {
             AVG_TRACE(Logger::WARNING, "Odd size for YCbCr bitmap.");
             m_Size.x++;
@@ -618,13 +704,13 @@ void Bitmap::allocBits()
 {
 //    cerr << "Bitmap::allocBits():" << m_Size <<  endl;
     m_Stride = getLineLen();
-    if (m_PF == YCbCr422) {
+    if (m_PF == YCbCr422 || m_PF == YCbCr420p) {
         if (m_Size.x%2 == 1) {
-            AVG_TRACE(Logger::WARNING, "Odd size for YCbCr bitmap.");
+            AVG_TRACE(Logger::WARNING, "Odd width for YCbCr bitmap.");
             m_Size.x++;
         }
         if (m_Size.y%2 == 1) {
-            AVG_TRACE(Logger::WARNING, "Odd size for YCbCr bitmap.");
+            AVG_TRACE(Logger::WARNING, "Odd height for YCbCr bitmap.");
             m_Size.y++;
         }
         //XXX: We allocate more than nessesary here because ffmpeg seems to
@@ -655,7 +741,41 @@ inline void YUVtoBGR32Pixel(Pixel32* pDest, int y, int u, int v)
     pDest->set(b,g,r,255);
 }
 
-void YUV422toBGR32Line(const unsigned char* pSrcLine, Pixel32 * pDestLine, int Width)
+void YUYV422toBGR32Line(const unsigned char* pSrcLine, Pixel32 * pDestLine, int Width)
+{
+    Pixel32 * pDestPixel = pDestLine;
+    
+    // We need the previous and next values to interpolate between the
+    // sampled u and v values.
+    int v = *(pSrcLine+3);
+    int v0; // Previous v
+    int u;
+    int u1; // Next u;
+    const unsigned char * pSrcPixels = pSrcLine;
+
+    for (int x = 0; x < Width/2-1; x++) {
+        // Two pixels at a time.
+        // Source format is YUYV.
+        u = pSrcPixels[1];
+        v0 = v;
+        v = pSrcPixels[3];
+        u1 = pSrcPixels[5];
+
+        YUVtoBGR32Pixel(pDestPixel, pSrcPixels[0], u, (v0+v)/2);
+        YUVtoBGR32Pixel(pDestPixel+1, pSrcPixels[2], (u+u1)/2, v);
+
+        pSrcPixels+=4;
+        pDestPixel+=2;
+    }
+    // Last pixels.
+    u = pSrcPixels[1];
+    v0 = v;
+    v = pSrcPixels[3];
+    YUVtoBGR32Pixel(pDestPixel, pSrcPixels[0], u, v0/2+v/2);
+    YUVtoBGR32Pixel(pDestPixel+1, pSrcPixels[2], u, v);
+}
+ 
+void UYVY422toBGR32Line(const unsigned char* pSrcLine, Pixel32 * pDestLine, int Width)
 {
     Pixel32 * pDestPixel = pDestLine;
     
@@ -735,10 +855,18 @@ void Bitmap::YCbCrtoBGR(const Bitmap& Orig)
     int Height = min(Orig.getSize().y, m_Size.y);
     int Width = min(Orig.getSize().x, m_Size.x);
     int StrideInPixels = m_Stride/getBytesPerPixel();
+    int TotalPixels = Width * Height;
     switch(Orig.m_PF) {
         case YCbCr422:
             for (int y=0; y<Height; ++y) {
-                YUV422toBGR32Line(pSrc, pDest, Width);
+                UYVY422toBGR32Line(pSrc, pDest, Width);
+                pDest += StrideInPixels;
+                pSrc += Orig.getStride();
+            }
+            break;
+        case YUYV422:
+            for (int y=0; y<Height; ++y) {
+                YUYV422toBGR32Line(pSrc, pDest, Width);
                 pDest += StrideInPixels;
                 pSrc += Orig.getStride();
             }
@@ -750,11 +878,106 @@ void Bitmap::YCbCrtoBGR(const Bitmap& Orig)
                 pSrc += Orig.getStride();
             }
             break;
+        case YCbCr420p: // this is a planar format
+            for (int y=0; y<Height; ++y) {
+                // Fast conversion algo with no interpolation
+                // TODO: interlacing aware interpolation
+                int lineoffs = Width * y;
+                int emioffs = (y/2) * (Width/2);
+                for (int x=0; x<Width; ++x) {
+                    // planes order: Y(1:1), Cb(1:4), Cr(1:4).
+                    // Offsets: Cb @ imgSize, Cr @ 5/4 * imgSize
+                    YUVtoBGR32Pixel(pDest + lineoffs + x,
+                        pSrc[lineoffs + x],
+                        pSrc[emioffs + x/2 + TotalPixels],
+                        pSrc[emioffs + x/2 + TotalPixels + TotalPixels/4]);
+                }
+            }
+            break;
         default:
             // This routine shouldn't be called with other pixel formats.
             assert(false);
     }
 }
+    
+void Bitmap::I16toI8(const Bitmap& Orig)
+{
+    assert(m_PF == I8);
+    assert(Orig.getPixelFormat() == I16);
+    const unsigned short * pSrc = (const unsigned short *)Orig.getPixels();
+    unsigned char * pDest = m_pBits;
+    int Height = min(Orig.getSize().y, m_Size.y);
+    int Width = min(Orig.getSize().x, m_Size.x);
+    int SrcStrideInPixels = Orig.getStride()/Orig.getBytesPerPixel();
+    for (int y=0; y<Height; ++y) {
+        const unsigned short * pSrcPixel = pSrc;
+        unsigned char * pDestPixel = pDest;
+        for (int x=0; x<Width; ++x) {
+            *pDestPixel++ = *pSrcPixel++ >> 8;
+        }
+        pDest += m_Stride;
+        pSrc += SrcStrideInPixels;
+    }
+}
+
+void Bitmap::I8toI16(const Bitmap& Orig)
+{
+    assert(m_PF == I16);
+    assert(Orig.getPixelFormat() == I8);
+    const unsigned char * pSrc = Orig.getPixels();
+    unsigned short * pDest = (unsigned short *)m_pBits;
+    int Height = min(Orig.getSize().y, m_Size.y);
+    int Width = min(Orig.getSize().x, m_Size.x);
+    int DestStrideInPixels = m_Stride/getBytesPerPixel();
+    for (int y=0; y<Height; ++y) {
+        const unsigned char * pSrcPixel = pSrc;
+        unsigned short * pDestPixel = pDest;
+        for (int x=0; x<Width; ++x) {
+            *pDestPixel++ = *pSrcPixel++ << 8;
+        }
+        pDest += DestStrideInPixels;
+        pSrc += Orig.getStride();
+    }
+}
+
+void Bitmap::I8toRGB(const Bitmap& Orig)
+{
+    assert(getBytesPerPixel() == 4 || getBytesPerPixel() == 3);
+    assert(Orig.getPixelFormat() == I8);
+    const unsigned char * pSrc = Orig.getPixels();
+    int Height = min(Orig.getSize().y, m_Size.y);
+    int Width = min(Orig.getSize().x, m_Size.x);
+    if (getBytesPerPixel() == 4) {
+        unsigned int * pDest = (unsigned int *)m_pBits;
+        int DestStrideInPixels = m_Stride/getBytesPerPixel();
+        for (int y=0; y<Height; ++y) {
+            const unsigned char * pSrcPixel = pSrc;
+            unsigned int * pDestPixel = pDest;
+            for (int x=0; x<Width; ++x) {
+                *pDestPixel = (((((255 << 8)+(*pSrcPixel)) << 8)+
+                        *pSrcPixel) << 8) +(*pSrcPixel);
+                pDestPixel ++;
+                pSrcPixel++;
+            }
+            pDest += DestStrideInPixels;
+            pSrc += Orig.getStride();
+        }
+    } else {
+        unsigned char * pDest = m_pBits;
+        for (int y=0; y<Height; ++y) {
+            const unsigned char * pSrcPixel = pSrc;
+            unsigned char * pDestPixel = pDest;
+            for (int x=0; x<Width; ++x) {
+                *pDestPixel++ = *pSrcPixel;
+                *pDestPixel++ = *pSrcPixel;
+                *pDestPixel++ = *pSrcPixel;
+                pSrcPixel++;
+            }
+            pDest += getStride();
+            pSrc += Orig.getStride();
+        }
+    }
+}
 
 template<class DestPixel, class SrcPixel>
 void createTrueColorCopy(Bitmap& Dest, const Bitmap & Src)
@@ -776,6 +999,31 @@ void createTrueColorCopy(Bitmap& Dest, const Bitmap & Src)
     }
 }
 
+template<>
+void createTrueColorCopy<Pixel32, Pixel8>(Bitmap& Dest, const Bitmap & Src)
+{
+    const unsigned char * pSrcLine = Src.getPixels();
+    unsigned char * pDestLine = Dest.getPixels();
+    int Height = min(Src.getSize().y, Dest.getSize().y);
+    int Width = min(Src.getSize().x, Dest.getSize().x);
+    int SrcStride = Src.getStride();
+    int DestStride = Dest.getStride();
+    for (int y = 0; y<Height; ++y) {
+        const unsigned char * pSrcPixel = pSrcLine;
+        unsigned char * pDestPixel = pDestLine;
+        for (int x = 0; x < Width; ++x) {
+            pDestPixel[0] =
+            pDestPixel[1] =
+            pDestPixel[2] = *pSrcPixel;
+            pDestPixel[3] = 255;
+            ++pSrcPixel;
+            pDestPixel+=4;
+        }
+        pSrcLine = pSrcLine + SrcStride;
+        pDestLine = pDestLine + DestStride;
+    }
+}
+
 template<class Pixel>
 void createTrueColorCopy(Bitmap& Dest, const Bitmap & Src)
 {
@@ -798,6 +1046,9 @@ void createTrueColorCopy(Bitmap& Dest, const Bitmap & Src)
         case R5G6B5:
             createTrueColorCopy<Pixel, Pixel16>(Dest, Src);
             break;
+        case I8:
+            createTrueColorCopy<Pixel, Pixel8>(Dest, Src);
+            break;
         default:
             // Unimplemented conversion.
             assert(false);
