@@ -22,11 +22,14 @@
 #ifndef _Bitmap_H_
 #define _Bitmap_H_
 
-#include "Point.h"
-#include "Rect.h"
+#include "../base/Point.h"
+#include "../base/Rect.h"
 
 #include <boost/shared_ptr.hpp>
+#include <stdlib.h>
 #include <string>
+#include <vector>
+#include <algorithm>
 
 namespace avg {
 
@@ -43,17 +46,21 @@ namespace avg {
 typedef enum {
     B5G6R5, B8G8R8, B8G8R8A8, B8G8R8X8, A8B8G8R8, X8B8G8R8,
     R5G6B5, R8G8B8, R8G8B8A8, R8G8B8X8, A8R8G8B8, X8R8G8B8,
-    I8, 
+    I8, I16, 
     YCbCr411,  // Interleaved YCbCr: Y,Y,Cb,Y,Y,Cr,... Effectively 12 bits per pixel.
-    YCbCr422,  // Interleaved YCbCr: Y,Cb,Y,Cr,...
+    YCbCr422,  // Interleaved YCbCr: Cb,Y,Cr,Y,...
+    YUYV422,   // Interleaved YCbCr, grey values first: Y,Cb,Y,Cr,...
     YCbCr420p, // Not really a valid pixel format. Signifies separate bitmaps
                // for Y, Cb and Cr components, with Cb and Cr half as big in 
                // both x and y dimensions. This is mpeg YCbCr, where the 
-               // color components have values from 16-235.
+               // color components have values from 16...235.
     YCbCrJ420p // Same as YCbCr420p, but this is the jpeg version with component
-               // values in the range 0.255
+               // values in the range 0...255
 } PixelFormat;
-    
+
+typedef std::vector<int> Histogram;
+typedef boost::shared_ptr<Histogram> HistogramPtr;
+
 class Bitmap
 {
 public:
@@ -61,6 +68,7 @@ public:
     Bitmap(IntPoint Size, PixelFormat PF, unsigned char * pBits, 
             int Stride, bool bCopyBits, const std::string& sName="");
     Bitmap(const Bitmap& Orig);
+    Bitmap(const Bitmap& Orig, bool bOwnsBits);
     Bitmap(Bitmap& Orig, const IntRect& Rect);
     Bitmap(const std::string& sURI);
     virtual ~Bitmap();
@@ -75,7 +83,7 @@ public:
     int getStride() const;
     PixelFormat getPixelFormat() const;
     void setPixelFormat(PixelFormat PF);
-    std::string getPixelFormatString();
+    std::string getPixelFormatString() const;
     static std::string getPixelFormatString(PixelFormat PF);
     unsigned char * getPixels();
     const unsigned char * getPixels() const;
@@ -89,6 +97,12 @@ public:
     int getLineLen() const;
     int getMemNeeded() const;
     bool hasAlpha() const;
+    HistogramPtr getHistogram(int Stride = 1) const;
+
+    template<class Pixel>
+    void setPixel(const IntPoint& p, Pixel Color);
+    template<class Pixel>
+    void drawLine(IntPoint p0, IntPoint p1, Pixel Color);
 
     void subtract(const Bitmap* pOtherBmp);
 
@@ -100,6 +114,9 @@ private:
     void initWithData(unsigned char * pBits, int Stride, bool bCopyBits);
     void allocBits();
     void YCbCrtoBGR(const Bitmap& Orig);
+    void I8toI16(const Bitmap& Orig);
+    void I8toRGB(const Bitmap& Orig);
+    void I16toI8(const Bitmap& Orig);
 
     IntPoint m_Size;
     int m_Stride;
@@ -111,5 +128,47 @@ private:
 
 typedef boost::shared_ptr<Bitmap> BitmapPtr;
 
+template<class Pixel>
+void Bitmap::setPixel(const IntPoint& p, Pixel Color)
+{
+    *(Pixel*)(&(m_pBits[p.y*m_Stride+p.x*getBytesPerPixel()])) = Color;
+}
+
+// TODO: This is slow, and it doesn't clip. Replace with external lib?
+template<class Pixel>
+void Bitmap::drawLine(IntPoint p0, IntPoint p1, Pixel Color)
+{
+    bool bSteep = abs(p1.y - p0.y) > abs(p1.x - p0.x);
+    if (bSteep) {
+        std::swap(p0.x, p0.y);
+        std::swap(p1.x, p1.y);
+    }
+    if (p0.x > p1.x) {
+        std::swap(p0, p1); 
+    }
+    int deltax = p1.x - p0.x;
+    int deltay = abs(p1.y - p0.y);
+    int error = -deltax/2;
+    int ystep;
+    int y = p0.y;
+    if (p0.y < p1.y) {
+        ystep = 1;
+    } else {
+        ystep = -1;
+    }
+    for (int x = p0.x; x <= p1.x; x++) {
+        if (bSteep) {
+            setPixel(IntPoint(y, x), Color); 
+        } else {
+            setPixel(IntPoint(x, y), Color);
+        }
+        error += deltay;
+        if (error > 0) {
+            y += ystep;
+            error -= deltax;
+        }
+    }
+}
+
 }
 #endif
