@@ -22,6 +22,8 @@
 #include "Filter.h"
 #include "Bitmap.h"
 
+#include "../base/ObjectCounter.h"
+
 #include <iostream>
 
 using namespace std;
@@ -30,14 +32,20 @@ namespace avg {
 
 Filter::Filter()
 {
+    ObjectCounter::get()->incRef(&typeid(*this));
 }
 
-void Filter::applyInPlace(BitmapPtr pBmp) const
+Filter::~Filter()
+{
+    ObjectCounter::get()->decRef(&typeid(*this));
+}
+
+void Filter::applyInPlace(BitmapPtr pBmp)
 {
     *pBmp = *(apply(pBmp));
 }
 
-BitmapPtr Filter::apply(BitmapPtr pBmpSource) const
+BitmapPtr Filter::apply(BitmapPtr pBmpSource)
 {
     BitmapPtr pBmpDest = BitmapPtr(new Bitmap(*pBmpSource));
     applyInPlace (pBmpDest);
