@@ -51,6 +51,7 @@ public:
     void operator +=(const Pixel32 Pix);
     void operator -=(const Pixel32 Pix);
     Pixel32 operator *(float f) const;
+    operator unsigned int() const;
 
     // Simple and fast 'distance' between two pixels. Just adds the
     // distances between the color components and treats colors
@@ -200,5 +201,10 @@ inline Pixel32 Pixel32::operator *(float f) const
           (unsigned char)(f*getG()), 
           (unsigned char)(f*getB()));
 }
+    
+inline Pixel32::operator unsigned int() const
+{
+    return *(unsigned int*)(m_Data);
+}
 
 #endif
