@@ -25,10 +25,13 @@
 #include "Node.h"
 #include "DisplayParams.h"
 
-#include "../graphics/Rect.h"
+#include "../video/IVideoDecoder.h"
+
 #include "../graphics/Pixel32.h"
 #include "../graphics/Bitmap.h"
 
+#include "../base/Rect.h"
+
 #include <string>
 
 namespace avg {
@@ -50,6 +53,7 @@ class DisplayEngine
         void deinitRender();
         void setFramerate(double rate);
         double getFramerate();
+        double getEffectiveFramerate();
         bool setVBlankRate(int rate);
         bool wasFrameLate();
         virtual double getRefreshRate() = 0;
@@ -58,6 +62,7 @@ class DisplayEngine
 
         virtual void render(AVGNodePtr pRootNode, bool bRenderEverything) = 0;
         void frameWait();
+        long long getDisplayTime();
         
         virtual void setClipRect() = 0;
         virtual bool pushClipRect(const DRect& rc, bool bClip) = 0;
@@ -80,9 +85,6 @@ class DisplayEngine
         virtual bool supportsBpp(int bpp) = 0; 
         // True if pixel order is RGB; BGR otherwise.
         virtual bool hasRGBOrdering() = 0; 
-        enum YCbCrMode {
-            NONE, OGL_MESA, OGL_APPLE, OGL_SHADER
-        };
         virtual YCbCrMode getYCbCrMode() = 0; 
         virtual void showCursor (bool bShow) = 0;
 
@@ -94,18 +96,24 @@ class DisplayEngine
     private:
         virtual bool initVBlank(int rate) = 0;
         virtual bool vbWait(int rate) = 0;
+
+        void calcEffFramerate();
         
         int m_NumFrames;
         int m_FramesTooLate;
         long long m_FrameWaitStartTime;
         long long m_TimeSpentWaiting;
         long long m_StartTime;
-        long long m_TargetTime;
-        long long m_LastFrameTime;
+        long long m_TargetTime;     // in microseconds.
+        long long m_LastFrameTime;  // in microseconds.
+        long long m_LastVideoFrameTime;  // measured by counting refreshes, in microseconds.
         int m_VBRate;
         double m_Framerate;
         bool m_bInitialized;
         bool m_bFrameLate;
+
+        long long m_StartFramerateCalcTime;
+        double m_EffFramerate;
 };
 
 }
