@@ -33,10 +33,6 @@ using namespace std;
 
 namespace avg {
 
-DivNode::DivNode()
-{
-}
-
 DivNode::DivNode (const xmlNodePtr xmlNode, Player * pPlayer)
     : Node(xmlNode, pPlayer)
 {
@@ -53,7 +49,6 @@ void DivNode::setDisplayEngine(DisplayEngine * pEngine)
     for  (int i = 0; i< (int)m_Children.size(); ++i) {
         m_Children[i]->setDisplayEngine(pEngine);
     }
-    
 }
 
 void DivNode::disconnect()
@@ -79,18 +74,28 @@ NodePtr DivNode::getChild (int i)
     return m_Children[i];
 }
 
-void DivNode::addChild (NodePtr pNewNode)
+void DivNode::appendChild (NodePtr pNewNode)
+{
+    insertChild(pNewNode, m_Children.size());
+}
+
+void DivNode::insertChild(NodePtr pNewNode, int i)
 {
     if (pNewNode->getState() == NS_CONNECTED) {
         throw(Exception(AVG_ERR_ALREADY_CONNECTED,
                 "Can't connect node with id "+pNewNode->getID()+
                 ": already connected."));
     }
-    m_Children.push_back(pNewNode);
+    if (i>m_Children.size()) {
+        throw(Exception(AVG_ERR_OUT_OF_RANGE,
+                pNewNode->getID()+"::insertChild: index out of bounds."));
+    }
+    std::vector<NodePtr>::iterator Pos = m_Children.begin()+i;
+    m_Children.insert(Pos, pNewNode);
     DivNodePtr Ptr = boost::dynamic_pointer_cast<DivNode>(getThis());
     pNewNode->setParent(Ptr);
     if (getState() == NS_CONNECTED) {
-        getPlayer()->addNodeID(pNewNode);
+        getPlayer()->registerNode(pNewNode);
     }
     if (isDisplayAvailable()) {
         pNewNode->setDisplayEngine(getEngine());
@@ -100,7 +105,6 @@ void DivNode::addChild (NodePtr pNewNode)
 void DivNode::removeChild (int i)
 {
     NodePtr pNode = getChild(i);
-    pNode->invalidate();
     pNode->setParent(DivNodePtr());
     pNode->disconnect();
     m_Children.erase(m_Children.begin()+i);
@@ -158,18 +162,6 @@ bool DivNode::obscures (const DRect& rect, int Child)
  
 }
 
-void DivNode::getDirtyRegion (Region& DirtyRegion)
-{
-    for (int i=0; i<getNumChildren(); i++){
-        Region ChildRegion;
-        getChild(i)->getDirtyRegion(ChildRegion);
-        DirtyRegion.addRegion(ChildRegion);
-    }
-    Region myRegion;
-    Node::getDirtyRegion(myRegion);
-    DirtyRegion.addRegion(myRegion);
-}
-
 string DivNode::getTypeStr ()
 {
     return "DivNode";
