@@ -24,7 +24,7 @@
 
 #include "Node.h"
 
-#include "../graphics/Point.h"
+#include "../base/Point.h"
 
 #include <string>
 
@@ -33,23 +33,22 @@ namespace avg {
 class DivNode : public Node
 {
 	public:
-        DivNode ();
         DivNode (const xmlNodePtr xmlNode, Player * pPlayer);
         virtual ~DivNode ();
         virtual void setDisplayEngine(DisplayEngine * pEngine);
         virtual void disconnect();
 
-        int getNumChildren ();
-        NodePtr getChild (int i);
-        void addChild (NodePtr newNode);
-        void removeChild (int i);
+        int getNumChildren();
+        NodePtr getChild(int i);
+        void appendChild(NodePtr pNewNode);
+        void insertChild(NodePtr pNewNode, int i);
+        void removeChild(int i);
         int indexOf(NodePtr pChild);
 
         virtual NodePtr getElementByPos (const DPoint & pos);
         virtual void prepareRender (int time, const DRect& parent);
         virtual void render (const DRect& rect);
         virtual bool obscures (const DRect& rect, int Child);
-        virtual void getDirtyRegion (Region& Dirtyregion);
         virtual std::string getTypeStr ();
 
         virtual std::string dump (int indent = 0);
@@ -58,7 +57,7 @@ class DivNode : public Node
         virtual Point<double> getPreferredMediaSize();	
     
     private:
-        std::vector <NodePtr> m_Children;
+        std::vector<NodePtr> m_Children;
 };
 
 }
