@@ -20,6 +20,7 @@
 //
 
 #include "Event.h"
+#include "Node.h"
 
 #include "../base/TimeSource.h"
 #include "../base/Logger.h"
@@ -27,13 +28,18 @@
 #include <iostream>
 #include <sstream>
 
+using namespace std;
+
 namespace avg {
 
 int Event::s_CurCounter = 0;
 
-Event::Event(Type type, int when)
-    : m_Type(type)
+Event::Event(Type type, Source source, int when)
+    : m_Type(type),
+      m_pNode(),
+      m_Source(source)
 {
+    ObjectCounter::get()->incRef(&typeid(*this));
     if (when == -1) {
         m_When = TimeSource::get()->getCurrentMillisecs();
     } else {
@@ -44,8 +50,15 @@ Event::Event(Type type, int when)
     m_Counter = s_CurCounter;    
 }
 
+Event::Event(const Event& e)
+{
+    ObjectCounter::get()->incRef(&typeid(*this));
+    *this = e;
+}
+
 Event::~Event()
 {
+    ObjectCounter::get()->decRef(&typeid(*this));
 }
 
 int Event::getWhen() const
@@ -58,6 +71,20 @@ Event::Type Event::getType() const
     return m_Type;
 }
 
+Event::Source Event::getSource() const
+{
+    return m_Source;
+}
+
+void Event::setElement(NodePtr pNode)
+{
+    m_pNode = pNode;
+}
+
+NodePtr Event::getElement() const
+{
+    return m_pNode;
+}
 void Event::trace()
 {
     switch(m_Type) {
@@ -67,20 +94,20 @@ void Event::trace()
         case KEYDOWN:
             AVG_TRACE(Logger::EVENTS, "KEYDOWN");
             break;
-        case MOUSEMOTION:
-            AVG_TRACE(Logger::EVENTS, "MOUSEMOTION");
+        case CURSORMOTION:
+            AVG_TRACE(Logger::EVENTS, "CURSORMOTION");
             break;
-        case MOUSEBUTTONUP:
-            AVG_TRACE(Logger::EVENTS, "MOUSEBUTTONUP");
+        case CURSORUP:
+            AVG_TRACE(Logger::EVENTS, "CURSORUP");
             break;
-        case MOUSEBUTTONDOWN:
-            AVG_TRACE(Logger::EVENTS, "MOUSEBUTTONDOWN");
+        case CURSORDOWN:
+            AVG_TRACE(Logger::EVENTS, "CURSORDOWN");
             break;
-        case MOUSEOVER:
-            AVG_TRACE(Logger::EVENTS, "MOUSEOVER");
+        case CURSOROVER:
+            AVG_TRACE(Logger::EVENTS, "CURSOROVER");
             break;
-        case MOUSEOUT:
-            AVG_TRACE(Logger::EVENTS, "MOUSEOUT");
+        case CURSOROUT:
+            AVG_TRACE(Logger::EVENTS, "CURSOROUT");
             break;
         case RESIZE:
             AVG_TRACE(Logger::EVENTS, "RESIZE");
@@ -88,6 +115,10 @@ void Event::trace()
         case QUIT:
             AVG_TRACE(Logger::EVENTS, "QUIT");
             break;
+        default:
+            AVG_TRACE(Logger::EVENTS, "UNKNOWN EVENT "<<m_Type);
+            break;
+
     }
 }
 
