@@ -23,38 +23,50 @@
 #define _Event_H_
 
 #include <functional>
+#include <boost/shared_ptr.hpp>
 #undef _POSIX_C_SOURCE
 
 namespace avg {
 
+class Node;
+typedef boost::shared_ptr<class Node> NodePtr;
 class Event {
     public:
         enum Type {
             KEYUP,
             KEYDOWN,
-            MOUSEMOTION,
-            MOUSEBUTTONUP,
-            MOUSEBUTTONDOWN,
-            MOUSEOVER,  
-            MOUSEOUT,
+            CURSORMOTION,
+            CURSORUP,
+            CURSORDOWN,
+            CURSOROVER,  
+            CURSOROUT,
             RESIZE,
             QUIT 
         };
+        enum Source {MOUSE, TOUCH, TRACK, NONE};
     
-        Event(Type type, int when=-1);
+        Event(Type type, Source source=NONE, int when=-1);
+        Event(const Event& e);
         virtual ~Event();
         
         virtual void trace();
 
         int getWhen() const;
         Type getType() const;
+        Event::Source getSource() const;
+        NodePtr getElement() const;
+        void setElement(NodePtr pNode);
         
         friend struct isEventAfter;
+        
+    protected:
+        Type m_Type;
+        NodePtr m_pNode;
+        int m_When;
 
     private:
-        int m_When;
-        Type m_Type;
         int m_Counter;
+        Source m_Source;
 
         static int s_CurCounter;
 };
