@@ -29,8 +29,8 @@ using namespace std;
 namespace avg {
 
     EventDispatcher::EventDispatcher()
-        : m_LastMouseEvent(MouseEvent(Event::MOUSEMOTION, false, false, false, 
-                0, 0, MouseEvent::NO_BUTTON))
+        : m_LastMouseEvent(MouseEvent(Event::CURSORMOTION, false, false, false, 
+                IntPoint(0, 0), MouseEvent::NO_BUTTON))
     {
     }
 
@@ -40,17 +40,19 @@ namespace avg {
 
     void EventDispatcher::dispatch() 
     {
+        vector<Event *> Events;
+
         for (unsigned int i = 0; i<m_EventSources.size(); ++i) {
             vector<Event*> curEvents = m_EventSources[i]->pollEvents();
-            for (unsigned int i= 0; i<curEvents.size(); i++) {
-                m_Events.push(curEvents[i]);
-            }
+            Events.insert(Events.end(), curEvents.begin(), curEvents.end());
         }
 
-        while (!m_Events.empty()) {
-            Event * pCurEvent = m_Events.top();
-            m_Events.pop();
-            sendEvent(pCurEvent);
+        vector<Event *>::iterator it;
+        for (it=Events.begin(); it != Events.end(); ++it) {
+            handleEvent(*it);
+        }
+        for (it=Events.begin(); it != Events.end(); ++it) {
+            delete *it;
         }
     }
 
@@ -71,6 +73,12 @@ namespace avg {
     }
 
     void EventDispatcher::sendEvent(Event* pEvent)
+    {
+        handleEvent(pEvent);
+        delete pEvent;
+    }
+
+    void EventDispatcher::handleEvent(Event* pEvent)
     {
         if (dynamic_cast<MouseEvent*>(pEvent) != 0) {
             m_LastMouseEvent = *(dynamic_cast<MouseEvent*>(pEvent));
