@@ -39,6 +39,7 @@ class EventDispatcher {
         EventDispatcher();
         virtual ~EventDispatcher();
         void dispatch();
+        //REFACTORME: this method should be in Player -- to say the least
         const MouseEvent& getLastMouseEvent() const;
         
         void addSource(IEventSource * pSource);
@@ -47,14 +48,14 @@ class EventDispatcher {
         void sendEvent(Event* pEvent);
 
     private:
+        void handleEvent(Event* pEvent);
+
         std::vector<IEventSource*> m_EventSources;
         std::vector<IEventSink*> m_EventSinks;
 
-        std::priority_queue<Event *,std::vector<Event *>,isEventAfter> 
-            m_Events;
-
         MouseEvent m_LastMouseEvent;
 };
+typedef boost::shared_ptr<EventDispatcher> EventDispatcherPtr;
 
 }
 
