@@ -35,7 +35,11 @@ class ISurface {
                 bool bFastDownload) = 0;
         virtual BitmapPtr lockBmp(int i=0) = 0;
         virtual void unlockBmps() {};
+        
+        virtual PixelFormat getPixelFormat() = 0;
+        virtual IntPoint getSize() = 0;
 
+        virtual bool wouldTile(IntPoint Size) { return false; }
 };
 
 }
