@@ -41,20 +41,13 @@ using namespace std;
 
 namespace avg {
 
-Image::Image ()
-    : m_Hue(-1),
-      m_Saturation(-1)
-{
-    m_pBmp = BitmapPtr(new Bitmap(IntPoint(1,1), R8G8B8));
-}
-
 Image::Image (const xmlNodePtr xmlNode, Player * pPlayer)
     : RasterNode(xmlNode, pPlayer)
 {
     m_href = getDefaultedStringAttr (xmlNode, "href", "");
     m_Hue = getDefaultedIntAttr (xmlNode, "hue", -1);
     m_Saturation = getDefaultedIntAttr (xmlNode, "saturation", -1);
-    m_pBmp = BitmapPtr(new Bitmap(IntPoint(1,1), R8G8B8));
+    m_pBmp = BitmapPtr(new Bitmap(IntPoint(1,1), R8G8B8X8));
     load();
 }
 
@@ -66,21 +59,29 @@ void Image::setDisplayEngine(DisplayEngine * pEngine)
 {
     RasterNode::setDisplayEngine(pEngine);
 
-    setupSurface();
+    setupSurface(&*m_pBmp);
 }
 
 void Image::disconnect()
 {
+/*
+    // Commenting this (and the corresponding line in setupSurface) out causes 
+    // a copy of the image to always be kept in main memory, so no readback has to
+    // take place.
+
     // Unload textures but keep bitmap in memory.
     ISurface * pSurface = getSurface();
-    m_pBmp = BitmapPtr(new Bitmap(*(pSurface->lockBmp())));
-    // XXX Yuck
+    BitmapPtr pSurfaceBmp = pSurface->lockBmp();
+    m_pBmp = BitmapPtr(new Bitmap(pSurfaceBmp->getSize(), pSurfaceBmp->getPixelFormat()));
+    m_pBmp->copyPixels(*pSurfaceBmp);
+    getSurface()->unlockBmps();
 #ifdef __i386__
+    // XXX Yuck
     if (!(getPlayer()->getDisplayEngine()->hasRGBOrdering())) {
         FilterFlipRGB().applyInPlace(m_pBmp);
     }
 #endif
-    getSurface()->unlockBmps();
+*/    
     RasterNode::disconnect();
 }
 
@@ -94,7 +95,7 @@ void Image::setHRef(const string& href)
     m_href = href;
     load();
     if (isDisplayAvailable()) {
-        setupSurface();
+        setupSurface(&*m_pBmp);
     }
     DPoint Size = getPreferredMediaSize();
     setViewport(-32767, -32767, Size.x, Size.y);
@@ -102,16 +103,41 @@ void Image::setHRef(const string& href)
 
 void Image::setBitmap(const Bitmap * pBmp)
 {
-    m_href = "";
-    
-    getSurface()->create(pBmp->getSize(), pBmp->getPixelFormat(), false);
-    getSurface()->lockBmp()->copyPixels(*pBmp);
+    // TODO: Add a unique bitmap identifier to the URI.
+    m_href = "mem://";
+    PixelFormat pf;
+    pf = R8G8B8X8;
+    if (pBmp->hasAlpha()) {
+        pf = R8G8B8A8;
+    }
+    if (pBmp->getPixelFormat() == I8) {
+        pf = I8;
+    }
+#ifdef __i386__
+    if (!(getPlayer()->getDisplayEngine()->hasRGBOrdering())) {
+        switch (pf) {
+            case R8G8B8X8:
+                pf = B8G8R8X8;
+                break;
+            case R8G8B8A8:
+                pf = B8G8R8A8;
+                break;
+            default:
+                break;
+        }
+    }
+#endif
+//    cerr << "pf: " << Bitmap::getPixelFormatString(pf) << endl;
+    getSurface()->create(pBmp->getSize(), pf, true);
+    BitmapPtr pSurfaceBmp = getSurface()->lockBmp();
+    pSurfaceBmp->copyPixels(*pBmp);
     getSurface()->unlockBmps();
     getEngine()->surfaceChanged(getSurface());
-    
+    DPoint Size = getPreferredMediaSize();
+    setViewport(-32767, -32767, Size.x, Size.y);
 }
 
-static ProfilingZone RenderProfilingZone("    Image::render");
+static ProfilingZone RenderProfilingZone("Image::render");
 
 void Image::render (const DRect& Rect)
 {
@@ -124,9 +150,10 @@ void Image::render (const DRect& Rect)
 
 bool Image::obscures (const DRect& Rect, int Child) 
 {
+    PixelFormat pf = getSurface()->getPixelFormat();
+    bool bHasAlpha = (pf == R8G8B8A8 || pf == B8G8R8A8);
     return (isActive() && getEffectiveOpacity() > 0.999
-            && !getSurface()->lockBmp()->hasAlpha() 
-            && getVisibleRect().Contains(Rect));
+            && !bHasAlpha && getVisibleRect().Contains(Rect));
 }
 
 string Image::getTypeStr ()
@@ -137,7 +164,7 @@ string Image::getTypeStr ()
 DPoint Image::getPreferredMediaSize()
 {
     if (isDisplayAvailable()) {
-        return DPoint(getSurface()->lockBmp()->getSize());
+        return DPoint(getSurface()->getSize());
     } else {
         return DPoint(m_pBmp->getSize());
     }
@@ -148,14 +175,14 @@ void Image::load()
     m_Filename = m_href;
     if (m_Filename != "") {
         initFilename(getPlayer(), m_Filename);
-        AVG_TRACE(Logger::PROFILE, "Loading " << m_Filename);
+        AVG_TRACE(Logger::MEMORY, "Loading " << m_Filename);
         try {
             m_pBmp = BitmapPtr(new Bitmap(m_Filename));
         } catch (Magick::Exception & ex) {
             AVG_TRACE(Logger::ERROR, ex.what());
         }
     } else {
-        m_pBmp = BitmapPtr(new Bitmap(IntPoint(1,1), R8G8B8));
+        m_pBmp = BitmapPtr(new Bitmap(IntPoint(1,1), R8G8B8X8));
     }
     if (m_Saturation != -1) {
         FilterColorize(m_Hue, m_Saturation).applyInPlace(
@@ -163,23 +190,30 @@ void Image::load()
     }
 }
 
-void Image::setupSurface()
+void Image::setupSurface(const Bitmap * pBmp)
 {
     PixelFormat pf;
-    pf = R8G8B8;
-    if (m_pBmp->hasAlpha()) {
+    pf = R8G8B8X8;
+    if (pBmp->hasAlpha()) {
         pf = R8G8B8A8;
     }
-    getSurface()->create(m_pBmp->getSize(), pf, false);
-    getSurface()->lockBmp()->copyPixels(*m_pBmp);
+    bool bUsePBO = true;
+#ifdef __APPLE__
+    if (getSurface()->wouldTile(pBmp->getSize())) {
+        bUsePBO = false;
+    }
+#endif
+    getSurface()->create(pBmp->getSize(), pf, bUsePBO);
+    BitmapPtr pSurfaceBmp = getSurface()->lockBmp();
+    pSurfaceBmp->copyPixels(*pBmp);
 #ifdef __i386__
     if (!(getPlayer()->getDisplayEngine()->hasRGBOrdering())) {
-        FilterFlipRGB().applyInPlace(getSurface()->lockBmp());
+        FilterFlipRGB().applyInPlace(pSurfaceBmp);
     }
 #endif
     getSurface()->unlockBmps();
     getEngine()->surfaceChanged(getSurface());
-    m_pBmp=BitmapPtr();
+//    m_pBmp=BitmapPtr();
 }
 
 }
