@@ -33,7 +33,6 @@ namespace avg {
 class Image : public RasterNode
 {
 	public:
-        Image ();
         Image (const xmlNodePtr xmlNode, Player * pPlayer);
         virtual ~Image ();
         virtual void setDisplayEngine(DisplayEngine * pEngine);
@@ -60,7 +59,7 @@ class Image : public RasterNode
 
     private:
         void load();
-        void setupSurface();
+        void setupSurface(const Bitmap * pBmp);
         std::string m_Filename;
         std::string m_href;
         BitmapPtr m_pBmp;
