@@ -23,6 +23,7 @@
 #include "DivNode.h"
 
 #include "Event.h"
+#include "CursorEvent.h"
 #include "MouseEvent.h"
 #include "DivNode.h"
 #include "Player.h"
@@ -31,8 +32,8 @@
 #include "../base/Logger.h"
 #include "../base/Exception.h"
 #include "../base/XMLHelper.h"
+#include "../base/ObjectCounter.h"
 
-#include <Python.h>
 #include <object.h>
 #include <compile.h>
 #include <eval.h>
@@ -46,27 +47,6 @@ using namespace std;
 
 namespace avg {
 
-Node::Node ()
-    : m_pParent(),
-      m_This(),
-      m_pEngine(0),
-      m_pPlayer(0),
-      m_ID(""),
-      m_MouseMoveHandler(""),
-      m_MouseButtonUpHandler(""),
-      m_MouseButtonDownHandler(""),
-      m_MouseOverHandler(""),
-      m_MouseOutHandler(""),
-      m_RelViewport(0,0,0,0),
-      m_AbsViewport(0,0,0,0),
-      m_Opacity(1.0),
-      m_bActive(true),
-      m_bSensitive(true),
-      m_WantedSize(0, 0)
-{
-    setState(NS_UNCONNECTED);
-}
-
 Node::Node (const xmlNodePtr xmlNode, Player * pPlayer)
     : m_pParent(),
       m_This(),
@@ -75,12 +55,13 @@ Node::Node (const xmlNodePtr xmlNode, Player * pPlayer)
       m_RelViewport(0,0,0,0),
       m_AbsViewport(0,0,0,0)
 {
+    ObjectCounter::get()->incRef(&typeid(*this));
     m_ID = getDefaultedStringAttr (xmlNode, "id", "");
-    m_MouseMoveHandler = getDefaultedStringAttr (xmlNode, "onmousemove", "");
-    m_MouseButtonUpHandler = getDefaultedStringAttr (xmlNode, "onmouseup", "");
-    m_MouseButtonDownHandler = getDefaultedStringAttr (xmlNode, "onmousedown", "");
-    m_MouseOverHandler = getDefaultedStringAttr (xmlNode, "onmouseover", "");
-    m_MouseOutHandler = getDefaultedStringAttr (xmlNode, "onmouseout", "");
+    addEventHandlers(Event::CURSORMOTION, getDefaultedStringAttr (xmlNode, "oncursormove", ""));
+    addEventHandlers(Event::CURSORUP, getDefaultedStringAttr (xmlNode, "oncursorup", ""));
+    addEventHandlers(Event::CURSORDOWN, getDefaultedStringAttr (xmlNode, "oncursordown", ""));
+    addEventHandlers(Event::CURSOROVER, getDefaultedStringAttr (xmlNode, "oncursorover", ""));
+    addEventHandlers(Event::CURSOROUT, getDefaultedStringAttr (xmlNode, "oncursorout", ""));
     m_RelViewport.tl.x = getDefaultedDoubleAttr (xmlNode, "x", 0.0);
     m_RelViewport.tl.y = getDefaultedDoubleAttr (xmlNode, "y", 0.0);
     m_WantedSize.x = getDefaultedDoubleAttr (xmlNode, "width", 0.0);
@@ -93,6 +74,11 @@ Node::Node (const xmlNodePtr xmlNode, Player * pPlayer)
 
 Node::~Node()
 {
+    EventHandlerMap::iterator it;
+    for (it=m_EventHandlerMap.begin(); it != m_EventHandlerMap.end(); ++it) {
+        Py_DECREF(it->second);
+    }
+    ObjectCounter::get()->decRef(&typeid(*this));
 }
 
 void Node::setThis(NodeWeakPtr This)
@@ -104,7 +90,7 @@ void Node::setParent(DivNodeWeakPtr pParent)
 {
     if (getParent() && !!(pParent.lock())) {
         throw(Exception(AVG_ERR_UNSUPPORTED, 
-                "Can't change parent of node."));
+                string("Can't change parent of node (") + m_ID + ")."));
     }
     m_pParent = pParent;
     setState(NS_CONNECTED);
@@ -129,7 +115,6 @@ void Node::setDisplayEngine(DisplayEngine * pEngine)
         pos = getParent()->getAbsViewport().tl;
     }
     m_AbsViewport = DRect (pos+getRelViewport().tl, pos+getRelViewport().br);
-    getPlayer()->addNodeID(getThis());
 }
 
 void Node::disconnect()
@@ -198,9 +183,6 @@ void Node::setOpacity(double opacity) {
     } else if (m_Opacity > 1.0) {
         m_Opacity = 1.0;
     }
-    if (m_bActive) {
-        invalidate();
-    }
 }
 
 bool Node::getActive() const {
@@ -211,7 +193,6 @@ void Node::setActive(bool bActive)
 {
     if (bActive != m_bActive) {
         m_bActive = bActive;
-        invalidate();
     }
 }
 
@@ -233,24 +214,42 @@ DivNodePtr Node::getParent() const
     }
 }
 
-double Node::getRelXPos(double x) 
+DPoint Node::getRelPos(const DPoint& AbsPos) const 
 {
     DRect VP = getAbsViewport();
-    return x-VP.tl.x;
+    return AbsPos-VP.tl;
 }
 
-double Node::getRelYPos(double y)
+void Node::setMouseEventCapture()
 {
-    DRect VP = getAbsViewport();
-    return y-VP.tl.y;
+    setEventCapture(MOUSECURSORID);
 }
 
-void Node::setEventCapture() {
-    m_pPlayer->setEventCapture(m_This);
+void Node::releaseMouseEventCapture()
+{
+    releaseEventCapture(MOUSECURSORID);
 }
 
-void Node::releaseEventCapture() {
-    m_pPlayer->releaseEventCapture(m_This);
+void Node::setEventCapture(int cursorID) {
+    m_pPlayer->setEventCapture(m_This, cursorID);
+}
+
+void Node::releaseEventCapture(int cursorID) {
+    m_pPlayer->releaseEventCapture(m_This, cursorID);
+}
+
+void Node::setEventHandler(Event::Type Type, Event::Source Source, PyObject * pFunc)
+{
+    EventHandlerID ID(Type, Source);
+    EventHandlerMap::iterator it = m_EventHandlerMap.find(ID);
+    if (it != m_EventHandlerMap.end()) {
+        Py_DECREF(it->second);
+        m_EventHandlerMap.erase(it);
+    }
+    if (pFunc != Py_None) {
+        Py_INCREF(pFunc);
+        m_EventHandlerMap[ID] = pFunc;
+    }
 }
 
 bool Node::isActive()
@@ -313,24 +312,6 @@ bool Node::obscures (const DRect& Rect, int Child)
     return false;
 }
 
-void Node::addDirtyRect(const DRect& Rect)
-{
-    m_DirtyRegion.addRect(Rect);
-}
-
-void Node::getDirtyRegion (Region& Region)
-{
-    Region.addRegion(m_DirtyRegion);
-    m_DirtyRegion.clear();
-}
-
-void Node::invalidate()
-{
-    if (isDisplayAvailable()) {
-        addDirtyRect(getVisibleRect());
-    }
-}
-
 Node::NodeState Node::getState() const
 {
     return m_State;
@@ -349,9 +330,6 @@ Player * Node::getPlayer() const
 
 void Node::setViewport (double x, double y, double width, double height)
 {
-    if (m_bActive) {
-        invalidate();
-    }
     if (x == -32767) {
         x = getRelViewport().tl.x;
     }
@@ -375,9 +353,6 @@ void Node::setViewport (double x, double y, double width, double height)
     }
     m_RelViewport = DRect (x, y, x+width, y+height);
     calcAbsViewport();
-    if (m_bActive) {
-        invalidate();
-    }
 }
 
 const DRect& Node::getRelViewport () const
@@ -458,64 +433,75 @@ string Node::getTypeStr () const
     return "Node";
 }
 
-void Node::handleMouseEvent (MouseEvent* pEvent)
+Node::EventHandlerID::EventHandlerID(Event::Type EventType, Event::Source Source)
+    : m_Type(EventType),
+      m_Source(Source)
 {
-    string Code;
-    int EventType = pEvent->getType();
-    switch (EventType) {
-        case Event::MOUSEMOTION:
-            Code = m_MouseMoveHandler;
-            break;
-        case Event::MOUSEBUTTONDOWN:
-            Code = m_MouseButtonDownHandler;
-            break;
-        case Event::MOUSEBUTTONUP:
-            Code = m_MouseButtonUpHandler;
-            break;
-        case Event::MOUSEOVER:
-            Code = m_MouseOverHandler;
-            break;
-        case Event::MOUSEOUT:
-            Code = m_MouseOutHandler;
-            break;
-         default:
-            break;
-    }
-    if (getID() != "" && EventType != Event::MOUSEMOTION) {
-        AVG_TRACE(Logger::EVENTS2, "Event handler: "+getID());
-    }
-    if (!Code.empty()) {
-        pEvent->setElement(m_This.lock());
-        callPython(Code, *pEvent);
-    }
-    if (getParent()) {
-        getParent()->handleMouseEvent (pEvent);
+}
+
+bool Node::EventHandlerID::operator < (const EventHandlerID& other) const {
+    if (m_Type == other.m_Type) {
+        return m_Source < other.m_Source;
+    } else {
+        return m_Type < other.m_Type;
     }
 }
 
-void Node::callPython (const string& Code, const Event& Event)
+void Node::handleEvent (Event* pEvent)
 {
-    // TODO:
-    //   - Pass Event to python.
-    //   - Handle python return code/exception and pass to caller.
-    PyObject * pModule = PyImport_AddModule("__main__");
-    if (!pModule) {
-        cerr << "Could not find module __main__." << endl;
-        exit(-1);
+    EventHandlerID ID(pEvent->getType(), pEvent->getSource());
+    EventHandlerMap::iterator it = m_EventHandlerMap.find(ID);
+    if (it!=m_EventHandlerMap.end()) {
+        pEvent->setElement(m_This.lock());
+        callPython(it->second, pEvent);
     }
-    PyObject * pDict = PyModule_GetDict(pModule);
-    PyObject * pFunc = PyDict_GetItemString(pDict, Code.c_str());
-    if (!pFunc) {
-        AVG_TRACE(Logger::ERROR, "Function \"" << Code << 
-                "\" not defined for node with id '"+m_ID+"'. Aborting.");
-        exit(-1);
+    if (getParent()) {
+        getParent()->handleEvent (pEvent);
     }
-    PyObject * pArgList = Py_BuildValue("()");
-    PyObject * pResult = PyObject_CallObject(pFunc, pArgList);
-    if (!pResult) {
-        throw error_already_set();
+}
+
+void Node::callPython (PyObject * pFunc, Event *pEvent)
+{
+    boost::python::call<void>(pFunc, boost::python::ptr(pEvent));
+}
+
+PyObject * Node::findPythonFunc(const string& Code)
+{
+    if (Code.empty()) {
+        return 0;
+    } else {
+        PyObject * pModule = PyImport_AddModule("__main__");
+        if (!pModule) {
+            cerr << "Could not find module __main__." << endl;
+            exit(-1);
+        }
+        PyObject * pDict = PyModule_GetDict(pModule);
+        PyObject * pFunc = PyDict_GetItemString(pDict, Code.c_str());
+        if (!pFunc) {
+            AVG_TRACE(Logger::ERROR, "Function \"" << Code << 
+                    "\" not defined for node with id '"+m_ID+"'. Aborting.");
+            exit(-1);
+        }
+        return pFunc;
+    }
+}
+
+void Node::addEventHandlers(Event::Type EventType, const string& Code)
+{
+    addEventHandler(EventType, Event::MOUSE, Code);
+    addEventHandler(EventType, Event::TOUCH, Code);
+    addEventHandler(EventType, Event::TRACK, Code);
+}
+
+void Node::addEventHandler(Event::Type EventType, Event::Source Source, 
+        const string& Code)
+{
+    PyObject * pFunc = findPythonFunc(Code);
+    if (pFunc) {
+        Py_INCREF(pFunc);
+        EventHandlerID ID(EventType, Source);
+        m_EventHandlerMap[ID] = pFunc;
     }
-    Py_DECREF(pArgList);
 }
 
 void Node::initFilename(Player * pPlayer, string& sFilename)
