@@ -23,28 +23,32 @@
 #define _Node_H_
 
 #include "Region.h"
+#include "Event.h"
 #include "ISurface.h"
-#include "../graphics/Point.h"
-#include "../graphics/Rect.h"
+
+#include "../base/Point.h"
+#include "../base/Rect.h"
 
 #include <libxml/parser.h>
 
 #include <boost/shared_ptr.hpp>
 #include <boost/weak_ptr.hpp>
 
+#include <Python.h>
+#undef HAVE_STAT
+
 #include <vector>
 #include <string>
+#include <map>
 
 namespace avg {
 
 class Node;
 class DivNode;
 class AVGNode;
-class Event;
 class Region;
 class DisplayEngine;
 class Player;
-class MouseEvent;
 class OGLSurface;
 
 typedef boost::shared_ptr<Node> NodePtr;
@@ -90,11 +94,13 @@ class Node
         void setSensitive(bool bSensitive);
 
         virtual DivNodePtr getParent() const;
-        double getRelXPos(double x);
-        double getRelYPos(double y);
+        DPoint getRelPos(const DPoint& AbsPos) const;
 
-        void setEventCapture();
-        void releaseEventCapture();
+        void setMouseEventCapture();
+        void releaseMouseEventCapture();
+        void setEventCapture(int cursorID);
+        void releaseEventCapture(int cursorID);
+        void setEventHandler(Event::Type Type, Event::Source Source, PyObject * pFunc);
 
         bool isActive();
         bool reactsToMouseEvents();
@@ -103,8 +109,6 @@ class Node
         virtual void maybeRender (const DRect& Rect);
         virtual void render (const DRect& Rect);
         virtual bool obscures (const DRect& Rect, int Child);
-        virtual void addDirtyRect(const DRect& Rect);
-        virtual void getDirtyRegion (Region& Region);
         virtual void setViewport (double x, double y, double width, 
                 double height);
         virtual const DRect& getRelViewport () const;
@@ -115,8 +119,7 @@ class Node
         virtual std::string dump (int indent = 0);
         virtual std::string getTypeStr () const;
         
-        virtual void handleMouseEvent (MouseEvent* pEvent); 
-        virtual void invalidate();
+        virtual void handleEvent (Event* pEvent); 
         NodeState getState() const;
         bool isDisplayAvailable() const;
         
@@ -125,7 +128,6 @@ class Node
                 NT_CAMERA, NT_DIV, NT_PANOIMAGE};
 
     protected:
-        Node ();
         Node (const xmlNodePtr xmlNode, Player * pPlayer);
         virtual DPoint getPreferredMediaSize() 
             { return DPoint(0,0); };
@@ -133,13 +135,17 @@ class Node
         DisplayEngine * getEngine() const;
         NodePtr getThis() const;
 
-        void callPython (const std::string& Code, const avg::Event& Event);
+        void callPython (PyObject * pFunc, avg::Event* pEvent);
+        void addEventHandlers(Event::Type EventType, const std::string& Code);
+        void addEventHandler(Event::Type EventType, Event::Source Source, 
+                const std::string& Code);
             
         void initFilename (Player * pPlayer, std::string& sFilename);
         void setState(NodeState State);
  
     private:
         void calcAbsViewport();
+        PyObject * findPythonFunc(const std::string& Code);
 
         DivNodeWeakPtr m_pParent;
         NodeWeakPtr m_This;
@@ -147,11 +153,17 @@ class Node
         Player * m_pPlayer;
 
         std::string m_ID;
-        std::string m_MouseMoveHandler;
-        std::string m_MouseButtonUpHandler;
-        std::string m_MouseButtonDownHandler;
-        std::string m_MouseOverHandler;
-        std::string m_MouseOutHandler;
+
+        struct EventHandlerID {
+            EventHandlerID(Event::Type EventType, Event::Source Source);
+
+            bool operator < (const EventHandlerID& other) const;
+
+            Event::Type m_Type;
+            Event::Source m_Source;
+        };
+        typedef std::map<EventHandlerID, PyObject *> EventHandlerMap;
+        EventHandlerMap m_EventHandlerMap;
 
         DRect m_RelViewport;      // In coordinates relative to the parent.
         DRect m_AbsViewport;      // In window coordinates.
@@ -162,7 +174,6 @@ class Node
         // Size specified by user.
         DPoint m_WantedSize;
 
-        Region m_DirtyRegion;
         NodeState m_State;
 };
 
