@@ -35,7 +35,6 @@
 #undef WARNING
 #endif
 #include <SDL/SDL.h>
-
 #include <iostream>
 #include <sstream>
 
@@ -65,6 +64,7 @@ namespace glproc {
     PFNGLBLENDEQUATIONPROC BlendEquation;
     PFNGLACTIVETEXTUREPROC ActiveTexture;
 #ifdef linux    
+    PFNGLXSWAPINTERVALSGIPROC SwapIntervalSGI;
     PFNGLXWAITVIDEOSYNCSGIPROC WaitVideoSyncSGI;
 #endif
 }    
@@ -161,18 +161,17 @@ GLfunction getFuzzyProcAddress(const char * psz)
     }
     return pProc;
 }
-
 #ifdef linux
 GLfunction getglXProcAddress(const char * psz)
 {
-    GLfunction pProc = (GLfunction)glXGetProcAddressARB((const GLubyte *)psz);
+    GLfunction pProc = (GLfunction)glXGetProcAddress((const GLubyte *)psz);
+    //GLfunction pProc = (GLfunction)glXGetProcAddressARB((const GLubyte *)psz);
     if (!pProc) {
         pProc = invalidGLCall;
     }
     return pProc;
 }
 #endif
-
 namespace glproc {
 
     void init() {
@@ -212,6 +211,7 @@ namespace glproc {
         BlendEquation = (PFNGLBLENDEQUATIONPROC)getFuzzyProcAddress("glBlendEquation");
         ActiveTexture = (PFNGLACTIVETEXTUREPROC)getFuzzyProcAddress("glActiveTexture");
 #ifdef linux
+        SwapIntervalSGI = (PFNGLXSWAPINTERVALSGIPROC)getglXProcAddress("glXSwapIntervalSGI");
         WaitVideoSyncSGI = (PFNGLXWAITVIDEOSYNCSGIPROC)getglXProcAddress("glXWaitVideoSyncSGI");
 #endif
     }
