@@ -48,7 +48,6 @@ bool queryGLXExtension(char *extName);
 
 enum OGLMemoryMode { 
     OGL,  // Standard OpenGL
-    MESA,
     PBO   // pixel buffer objects
 };
 
@@ -78,6 +77,7 @@ namespace glproc {
     extern PFNGLACTIVETEXTUREPROC ActiveTexture;
 
 #ifdef linux
+    extern PFNGLXSWAPINTERVALSGIPROC SwapIntervalSGI;
     extern PFNGLXWAITVIDEOSYNCSGIPROC WaitVideoSyncSGI;
 #endif
 
