@@ -23,10 +23,12 @@
 #include "OGLHelper.h"
 #include "Player.h"
 #include "MathHelper.h"
+
 #include "../base/Logger.h"
 #include "../base/Exception.h"
 #include "../base/ScopeTimer.h"
-#include "../graphics/Point.h"
+#include "../base/ObjectCounter.h"
+#include "../base/Point.h"
 
 #include <iostream>
 #include <sstream>
@@ -43,6 +45,7 @@ OGLSurface::OGLSurface(SDLDisplayEngine * pEngine)
       m_NumHorizTextures(-1),
       m_NumVertTextures(-1)
 {
+    ObjectCounter::get()->incRef(&typeid(*this));
     // Do an NVIDIA texture support query if it hasn't happened already.
 //    getTextureMode();
 }
@@ -65,11 +68,12 @@ OGLSurface::~OGLSurface()
         default:
             break;
     }
+    ObjectCounter::get()->decRef(&typeid(*this));
 }
 
 void OGLSurface::create(const IntPoint& Size, PixelFormat pf, bool bFastDownload)
 {
-//    cerr << "create: " << Bitmap::getPixelFormatString(pf) << endl;
+//    cerr << "create: " << Size << ", " << Bitmap::getPixelFormatString(pf) << endl;
     
     if (m_bBound && m_Size == Size && m_pf == pf) {
         // If nothing's changed, we can ignore everything.
@@ -92,7 +96,7 @@ void OGLSurface::create(const IntPoint& Size, PixelFormat pf, bool bFastDownload
         
     unbind();
     setupTiles();
-    initTileVertices();
+    initTileVertices(m_TileVertices);
 }
 
 void OGLSurface::createFromBits(IntPoint Size, PixelFormat pf,
@@ -172,59 +176,47 @@ void OGLSurface::setMaxTileSize(const IntPoint& MaxTileSize)
     }
     if (m_pBmps[0]) {
         setupTiles();
-        initTileVertices();
+        initTileVertices(m_TileVertices);
     }
 }
 
-int OGLSurface::getNumVerticesX()
-{
-    return m_NumHorizTextures+1;
-}
-
-int OGLSurface::getNumVerticesY() 
-{
-    return m_NumVertTextures+1;
-}
-
-DPoint OGLSurface::getOrigVertexCoord(int x, int y)
+VertexGrid OGLSurface::getOrigVertexCoords()
 {
     if (!m_bBound) {
         bind();
     }
-    if (x < 0 || x > m_NumHorizTextures || y < 0 || y > m_NumVertTextures) {
-        AVG_TRACE(Logger::WARNING, 
-                "getOrigVertexCoord called, but coordinate out of bounds.");
-        return DPoint(0,0);
-    }
-    DPoint Vertex;
-    initTileVertex(x, y, Vertex);
-    return Vertex;
+    VertexGrid Grid;
+    initTileVertices(Grid);
+    return Grid;
 }
 
-DPoint OGLSurface::getWarpedVertexCoord(int x, int y)
+VertexGrid OGLSurface::getWarpedVertexCoords()
 {
     if (!m_bBound) {
         bind();
     }
-    if (x < 0 || x > m_NumHorizTextures || y < 0 || y > m_NumVertTextures) {
-        AVG_TRACE(Logger::WARNING, 
-                "getWarpedVertexCoord called, but coordinate out of bounds.");
-        return DPoint(0,0);
-    }
-    return m_TileVertices[y][x];
+    return m_TileVertices;
 }
 
-void OGLSurface::setWarpedVertexCoord(int x, int y, const DPoint& Vertex)
+void OGLSurface::setWarpedVertexCoords(const VertexGrid& Grid)
 {
     if (!m_bBound) {
         bind();
     }
-    if (x < 0 || x > m_NumHorizTextures || y < 0 || y > m_NumVertTextures) {
-        AVG_TRACE(Logger::WARNING, 
-                "setWarpedVertexCoord called, but coordinate out of bounds.");
-        return;
+    bool bGridOK = true;
+    if (Grid.size() != (unsigned)(m_NumVertTextures+1)) {
+        bGridOK = false;
     }
-    m_TileVertices[y][x] = Vertex;
+    for (unsigned i = 0; i< Grid.size(); ++i) {
+        if (Grid[i].size() != (unsigned)(m_NumHorizTextures+1)) {
+            bGridOK = false;
+        }
+    }
+    if (!bGridOK) {
+        throw Exception(AVG_ERR_OUT_OF_RANGE, 
+                "setWarpedVertexCoords() called with incorrect grid size.");
+    }
+    m_TileVertices = Grid;
 }
 
 string getGlModeString(int Mode) 
@@ -251,7 +243,6 @@ string getGlModeString(int Mode)
 
 void OGLSurface::bind() 
 {
-//    cerr << "OGLSurface::bind()" << endl;
     if (m_bBound) {
         rebind();
     } else {
@@ -358,6 +349,16 @@ void OGLSurface::rebind()
     }
 }
 
+PixelFormat OGLSurface::getPixelFormat()
+{
+    return m_pf;
+}
+        
+IntPoint OGLSurface::getSize()
+{
+    return m_Size;
+}
+
 void OGLSurface::blt(const DRect* pDestRect, 
         double angle, const DPoint& pivot, 
         DisplayEngine::BlendMode Mode)
@@ -369,6 +370,19 @@ void OGLSurface::blt(const DRect* pDestRect,
     bltTexture(pDestRect, angle, pivot, Mode);
 }
 
+bool OGLSurface::wouldTile(IntPoint Size)
+{
+    if (m_MaxTileSize.x != -1 || m_MaxTileSize.y != -1 ||
+        Size.x > m_pEngine->getMaxTexSize() || 
+        Size.y > m_pEngine->getMaxTexSize() ||
+        m_pEngine->getTextureMode() == GL_TEXTURE_2D)
+    {
+        return true;
+    } else {
+        return false;
+    }
+}
+
 void OGLSurface::setupTiles()
 {
     if (m_Size.x > m_pEngine->getMaxTexSize() || 
@@ -399,14 +413,14 @@ void OGLSurface::setupTiles()
 
 }
 
-void OGLSurface::initTileVertices()
+void OGLSurface::initTileVertices(VertexGrid& Grid)
 {
     std::vector<DPoint> TileVerticesLine(m_NumHorizTextures+1);
-    m_TileVertices = std::vector<std::vector<DPoint> >
+    Grid = std::vector<std::vector<DPoint> >
                 (m_NumVertTextures+1, TileVerticesLine);
-    for (unsigned int y=0; y<m_TileVertices.size(); y++) {
-        for (unsigned int x=0; x<m_TileVertices[y].size(); x++) {
-            initTileVertex(x, y, m_TileVertices[y][x]);
+    for (unsigned int y=0; y<Grid.size(); y++) {
+        for (unsigned int x=0; x<Grid[y].size(); x++) {
+            initTileVertex(x, y, Grid[y][x]);
         }
     }
 }
@@ -458,6 +472,7 @@ void OGLSurface::createBitmap(const IntPoint& Size, PixelFormat pf, int i)
 
 void OGLSurface::unlockBmp(int i) 
 {
+//    cerr << "unlockBmp" << endl;
     switch (m_MemoryMode) {
         case PBO:
             glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, m_hPixelBuffers[i]);
