@@ -26,15 +26,18 @@
 #include "OGLTile.h"
 #include "OGLHelper.h"
 #include "OGLShader.h"
-
 #include "SDLDisplayEngine.h"
-#include "../graphics/Rect.h"
+
+#include "../base/Point.h"
+#include "../base/Rect.h"
 
 #include <vector>
 #include <string>
 
 namespace avg {
 
+typedef std::vector<std::vector<DPoint> > VertexGrid;
+
 class OGLSurface: public ISurface {
     public:
         OGLSurface(SDLDisplayEngine * pEngine);
@@ -54,20 +57,23 @@ class OGLSurface: public ISurface {
         void unbind();
         void rebind();
 
+        PixelFormat getPixelFormat();
+        IntPoint getSize();
+
         void blt(const DRect* pDestRect,
                 double angle, const DPoint& pivot, 
                 DisplayEngine::BlendMode Mode);
 
         void setMaxTileSize(const Point<int>& MaxTileSize);
-        int getNumVerticesX();
-        int getNumVerticesY();
-        DPoint getOrigVertexCoord(int x, int y);
-        DPoint getWarpedVertexCoord(int x, int y);
-        void setWarpedVertexCoord(int x, int y, const DPoint& Vertex);
+        VertexGrid getOrigVertexCoords();
+        VertexGrid getWarpedVertexCoords();
+        void setWarpedVertexCoords(const VertexGrid& Grid);
  
+        bool wouldTile(IntPoint Size);
+
     private:
         void setupTiles();
-        void initTileVertices();
+        void initTileVertices(VertexGrid& Grid);
         void initTileVertex (int x, int y, DPoint& Vertex);
 
         void createBitmap(const IntPoint& Size, PixelFormat pf, int index);
@@ -92,7 +98,7 @@ class OGLSurface: public ISurface {
         int m_NumHorizTextures;
         int m_NumVertTextures;
         std::vector<std::vector<OGLTilePtr> > m_pTiles;
-        std::vector<std::vector<DPoint> > m_TileVertices;
+        VertexGrid m_TileVertices;
 
         OGLMemoryMode m_MemoryMode;
 
