@@ -23,6 +23,7 @@
 #include "../base/Logger.h"
 #include "../base/Exception.h"
 #include "../base/ScopeTimer.h"
+#include "../base/ObjectCounter.h"
 
 #include <iostream>
 #include <string>
@@ -38,6 +39,7 @@ OGLTile::OGLTile(IntRect Extent, IntPoint TexSize, int Stride, PixelFormat pf,
       m_pf(pf),
       m_pEngine(pEngine)
 {
+    ObjectCounter::get()->incRef(&typeid(*this));
     if (m_pf == YCbCr420p || m_pf == YCbCrJ420p) {
         createTexture(0, m_TexSize, Stride, I8);
         createTexture(1, m_TexSize/2, Stride/2, I8);
@@ -50,13 +52,12 @@ OGLTile::OGLTile(IntRect Extent, IntPoint TexSize, int Stride, PixelFormat pf,
 OGLTile::~OGLTile()
 {
     if (m_pf == YCbCr420p || m_pf == YCbCrJ420p) {
-        glDeleteTextures(1, &m_TexID[0]);
-        glDeleteTextures(2, &m_TexID[0]);
-        glDeleteTextures(3, &m_TexID[0]);
+        glDeleteTextures(3, m_TexID);
     } else {
-        glDeleteTextures(1, &m_TexID[0]);
+        glDeleteTextures(1, m_TexID);
     }
-    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "OGLTile::~OGLTile: glDeleteTextures()");
+    OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "OGLTile::~OGLTile: glDeleteTextures()");    
+    ObjectCounter::get()->decRef(&typeid(*this));
 }
 
 const IntRect& OGLTile::getExtent() const
@@ -111,7 +112,7 @@ void OGLTile::blt(const DPoint& TLPoint, const DPoint& TRPoint,
     } else {
         glproc::ActiveTexture(GL_TEXTURE0);
         glBindTexture(TextureMode, m_TexID[0]);
-        if (m_pEngine->getYCbCrMode() == SDLDisplayEngine::OGL_SHADER) {
+        if (m_pEngine->getYCbCrMode() == OGL_SHADER) {
             glproc::UseProgramObject(0);
         }
     }
@@ -182,7 +183,7 @@ void OGLTile::createTexture(int i, IntPoint Size, int Stride, PixelFormat pf)
     }
 }
 
-static ProfilingZone TexSubImageProfilingZone("    OGLTile::texture download");
+static ProfilingZone TexSubImageProfilingZone("OGLTile::texture download");
 
 void OGLTile::downloadTexture(int i, BitmapPtr pBmp, int stride, 
                 OGLMemoryMode MemoryMode) const
