@@ -19,14 +19,10 @@
 //  Current versions can be found at www.libavg.de
 //
 
-#include "../avgconfig.h"
-
 #include "PanoImage.h"
 #include "SDLDisplayEngine.h"
 #include "MathHelper.h"
-#ifdef AVG_ENABLE_GL
 #include "OGLHelper.h"
-#endif
 
 #include "../base/Logger.h"
 #include "../base/ProfilingZone.h"
@@ -49,18 +45,6 @@ const int TEX_WIDTH = 64;
 
 namespace avg {
 
-PanoImage::PanoImage ()
-    : m_Filename(""),
-      m_SensorWidth(0),
-      m_SensorHeight(0),
-      m_FocalLength(0),
-      m_Rotation(-1),
-      m_Hue(-1),
-      m_Saturation(-1)
-{
-    m_pBmp = BitmapPtr(new Bitmap(IntPoint(1,1), R8G8B8));
-}
-
 PanoImage::PanoImage (const xmlNodePtr xmlNode, Player * pPlayer)
     : Node (xmlNode, pPlayer)
 {
@@ -84,19 +68,6 @@ PanoImage::~PanoImage ()
 
 void PanoImage::setDisplayEngine (DisplayEngine * pEngine)
 {
-    SDLDisplayEngine * pSDLEngine;
-#ifdef AVG_ENABLE_GL    
-    pSDLEngine = dynamic_cast<SDLDisplayEngine*>(pEngine);
-#else
-    pSDLEngine = 0;
-#endif
-    if (!pSDLEngine) {
-        AVG_TRACE(Logger::ERROR,
-                "Panorama images are only allowed when "
-                "the display engine is OpenGL. Aborting.");
-        // TODO: Disable image.
-        exit(-1);
-    }
     Node::setDisplayEngine(pEngine);
     
     setupTextures();
@@ -108,12 +79,10 @@ void PanoImage::disconnect()
     Node::disconnect();
 }
 
-static ProfilingZone PanoRenderProfilingZone("    PanoImage::render");
-//static ProfilingZone PanoRenderQuadsProfilingZone("PanoImage::render quads");
+static ProfilingZone PanoRenderProfilingZone("PanoImage::render");
 
 void PanoImage::render(const DRect& Rect)
 {
-#ifdef AVG_ENABLE_GL
     ScopeTimer Timer(PanoRenderProfilingZone);
     glPushMatrix();
     OGLErrorCheck(AVG_ERR_VIDEO_GENERAL,
@@ -171,7 +140,6 @@ void PanoImage::render(const DRect& Rect)
     double HorizOffset = m_Rotation+m_fovy*m_aspect/2;
 //    glutWireSphere(1, 20, 16);
     for (unsigned int i=0; i<m_TileTextureIDs.size(); ++i) {
-//        ScopeTimer ScopeTimer(PanoRenderQuadsProfilingZone);
         unsigned int TexID = m_TileTextureIDs[i];
         glBindTexture(GL_TEXTURE_2D, TexID);
         OGLErrorCheck(AVG_ERR_VIDEO_GENERAL,
@@ -218,7 +186,6 @@ void PanoImage::render(const DRect& Rect)
     glPopMatrix();
     glMatrixMode(GL_MODELVIEW);
     glPopMatrix();
-#endif    
 }
 
 bool PanoImage::obscures (const DRect& Rect, int Child)
@@ -337,7 +304,7 @@ DPoint PanoImage::getPreferredMediaSize()
 void PanoImage::load()
 {
     m_Filename = m_href;
-//    AVG_TRACE(Logger::PROFILE, "Loading " << m_Filename);
+    AVG_TRACE(Logger::MEMORY, "Loading " << m_Filename);
     if (m_Filename != "") {
         initFilename(getPlayer(), m_Filename);
         try {
@@ -363,7 +330,6 @@ void PanoImage::setupTextures()
     if (getEngine()->hasRGBOrdering()) {
         FilterFlipRGB().applyInPlace(m_pBmp);
     }
-#ifdef AVG_ENABLE_GL
     if (!m_TileTextureIDs.empty()) {
         clearTextures();
     }
@@ -429,7 +395,6 @@ void PanoImage::setupTextures()
         OGLErrorCheck(AVG_ERR_VIDEO_GENERAL,
                 "PanoImage::setupTextures: glTexSubImage2D()");
    }
-#endif
 }
 
 void PanoImage::clearTextures()
