@@ -22,13 +22,7 @@
 #ifndef _PanoImage_H_
 #define _PanoImage_H_
 
-#include "../avgconfig.h"
-#undef PACKAGE
-#undef PACKAGE_BUGREPORT
-#undef PACKAGE_NAME
-#undef PACKAGE_STRING
-#undef PACKAGE_TARNAME
-#undef PACKAGE_VERSION
+#include "../avgconfigwrapper.h"
 
 #include "Node.h"
 #ifdef AVG_ENABLE_GL
@@ -46,7 +40,6 @@ class SDLDisplayEngine;
 class PanoImage : public Node
 {
 	public:
-        PanoImage();
         PanoImage(const xmlNodePtr xmlNode, Player * pPlayer);
         virtual ~PanoImage();
         
