@@ -19,35 +19,24 @@
 //  Current versions can be found at www.libavg.de
 //
 
-#include "../avgconfig.h"
-#undef PACKAGE
-#undef PACKAGE_BUGREPORT
-#undef PACKAGE_NAME
-#undef PACKAGE_STRING
-#undef PACKAGE_TARNAME
-#undef PACKAGE_VERSION
-
 #include "Player.h"
 
+#include "../avgconfig.h"
 #include "avgdtd.h"
 #include "AVGNode.h"
 #include "DivNode.h"
 #include "Words.h"
 #include "Video.h"
-#include "Camera.h"
+#include "CameraNode.h"
 #include "Image.h"
 #include "PanoImage.h"
 
+#include "TrackerEventSource.h"
 #include "Event.h"
 #include "MouseEvent.h"
 #include "KeyEvent.h"
 
-#ifdef AVG_ENABLE_DFB
-#include "DFBDisplayEngine.h"
-#endif
-#ifdef AVG_ENABLE_GL
 #include "SDLDisplayEngine.h"
-#endif
 
 #include "../base/FileHelper.h"
 #include "../base/Exception.h"
@@ -57,6 +46,14 @@
 #include "../base/Profiler.h"
 #include "../base/ScopeTimer.h"
 #include "../base/TimeSource.h"
+#include "../base/MemHelper.h"
+#include "../base/Rect.h"
+
+#include "../imaging/FWCamera.h"
+#ifdef AVG_ENABLE_V4L2
+#include "../imaging/V4LCamera.h"
+#endif
+#include "../imaging/FakeCamera.h"
 
 #include <Magick++.h>
 
@@ -71,6 +68,7 @@
 #include <iostream>
 #include <sstream>
 #include <assert.h>
+#include <math.h>
 
 using namespace std;
 
@@ -80,21 +78,30 @@ Player::Player()
     : m_pRootNode(),
       m_pDisplayEngine(0),
       m_TestHelper(this),
+      m_pTracker(0),
       m_bInHandleTimers(false),
       m_bCurrentTimeoutDeleted(false), 
       m_pLastMouseNode(),
       m_pEventCaptureNode(),
-      m_bIsPlaying(false)
+      m_bUseFakeCamera(false),
+      m_bIsPlaying(false),
+      m_bFakeFPS(false),
+      m_FakeFPS(0),
+      m_FrameTime(0),
+      m_PlayStartTime(0)
 {
+    ThreadProfilerPtr pThreadProfiler = ThreadProfilerPtr(new ThreadProfiler("Main"));
+    Profiler::get().registerThreadProfiler(pThreadProfiler);
     initConfig();
     // Find and parse dtd.
-    registerDTDEntityLoader(g_pAVGDTD);
+    registerDTDEntityLoader("avg.dtd", g_pAVGDTD);
     string sDTDFName = "avg.dtd";
     m_dtd = xmlParseDTD(NULL, (const xmlChar*) sDTDFName.c_str());
     if (!m_dtd) {
         AVG_TRACE(Logger::WARNING, 
                 "DTD not found at " << sDTDFName << ". Not validating xml files.");
     }
+    m_pLastMouseNode[MOUSECURSORID]=NodePtr();
 }
 
 Player::~Player()
@@ -102,25 +109,11 @@ Player::~Player()
     if (m_pDisplayEngine) {
         delete m_pDisplayEngine;
     }
-}
-
-void Player::setDisplayEngine(DisplayEngineType engine)
-{
-    if (m_pRootNode) {
-        throw Exception(AVG_ERR_VIDEO_INIT_FAILED,
-                "Player::setDisplayEngine called after loadFile.");
+    if (m_pTracker) {
+        delete m_pTracker;
     }
-    switch (engine) {
-        case DFB:
-            m_sDisplaySubsystem="DFB";
-            break;
-        case OGL:
-            m_sDisplaySubsystem="OGL";
-            break;
-        default:
-            AVG_TRACE(Logger::ERROR,
-                    "Unknown display engine type in setDisplayEngine. Aborting.");
-            exit(-1);
+    if (m_dtd) {
+        xmlFreeDtd(m_dtd);
     }
 }
 
@@ -139,12 +132,11 @@ void Player::setResolution(bool bFullscreen,
     }
 }
         
-void Player::setOGLOptions(bool bUsePOW2Textures, DisplayEngine::YCbCrMode DesiredYCbCrMode, 
-                bool bUseRGBOrder, bool bUsePixelBuffers, int MultiSampleSamples)
+void Player::setOGLOptions(bool bUsePOW2Textures, YCbCrMode DesiredYCbCrMode, 
+                bool bUsePixelBuffers, int MultiSampleSamples)
 {
     m_bUsePOW2Textures = bUsePOW2Textures;
     m_YCbCrMode = DesiredYCbCrMode;
-    m_bUseRGBOrder = bUseRGBOrder;
     m_bUsePixelBuffers = bUsePixelBuffers;
     m_MultiSampleSamples = MultiSampleSamples;
 }
@@ -152,7 +144,8 @@ void Player::setOGLOptions(bool bUsePOW2Textures, DisplayEngine::YCbCrMode Desir
 void Player::loadFile (const std::string& filename)
 {
     try {
-        AVG_TRACE(Logger::PROFILE, 
+        m_pEventDispatcher = EventDispatcherPtr(new EventDispatcher);
+        AVG_TRACE(Logger::MEMORY, 
                 std::string("Player::LoadFile(") + filename + ")");
         if (m_pRootNode) {
             cleanup();
@@ -176,6 +169,10 @@ void Player::loadFile (const std::string& filename)
         xmlDoValidityCheckingDefaultValue =0;
 
         xmlDocPtr doc;
+        if (!fileExists(RealFilename)) {
+            throw (Exception(AVG_ERR_FILEIO, 
+                        string("File ")+RealFilename+" not found."));
+        }
         doc = xmlParseFile(RealFilename.c_str());
         if (!doc) {
             throw (Exception(AVG_ERR_XML_PARSE, 
@@ -223,21 +220,24 @@ void Player::play()
         assert(m_pRootNode);
         initGraphics();
         m_pRootNode->setDisplayEngine(m_pDisplayEngine);
-        
-        m_EventDispatcher.addSource(m_pEventSource);
-        m_EventDispatcher.addSource(&m_TestHelper);
-        m_EventDispatcher.addSink(&m_EventDumper);
-        m_EventDispatcher.addSink(this);
+       
+        m_pEventDispatcher->addSource(m_pEventSource);
+        m_pEventDispatcher->addSource(&m_TestHelper);
+        m_pEventDispatcher->addSink(&m_EventDumper);
+        m_pEventDispatcher->addSink(this);
         
         m_pDisplayEngine->initRender();
         m_bStopping = false;
 
-        Profiler::get().start();
+        m_PlayStartTime = TimeSource::get()->getCurrentMillisecs();
+        m_FrameTime = 0;
+        m_NumFrames = 0;
         m_pDisplayEngine->render(m_pRootNode, true);
         if (m_pDisplayEngine->wasFrameLate()) {
-            Profiler::get().dumpFrame();
+            ThreadProfiler::get()->dumpFrame();
         }
         
+        ThreadProfiler::get()->start();
         try {
             while (!m_bStopping) {
                 doFrame();
@@ -289,15 +289,78 @@ bool Player::setVBlankFramerate(int rate) {
         return true;
     }
 }
+        
+double Player::getEffectiveFramerate() {
+    if (m_bIsPlaying) {
+        return m_pDisplayEngine->getEffectiveFramerate();
+    } else {
+        return 0;
+    }
+}
+
+unsigned Player::getMemUsed()
+{
+    return avg::getMemUsed();
+}
 
 TestHelper * Player::getTestHelper()
 {
     return &m_TestHelper;
 }
 
+void Player::setFakeFPS(double fps)
+{
+    if (fabs(fps + 1.0) < 0.0001) {
+        // fps = -1
+        m_bFakeFPS = false;
+    } else {
+        m_bFakeFPS = true;
+        m_FakeFPS = fps;
+    }
+}
+
+long long Player::getFrameTime()
+{
+    return m_FrameTime;
+}
+
+TrackerEventSource * Player::addTracker()
+{
+    TrackerConfig Config;
+    Config.load();
+    CameraPtr pCamera;
+    
+    if (Config.m_sSource == "v4l") {
+#ifdef AVG_ENABLE_V4L2
+        AVG_TRACE(Logger::CONFIG, "Adding a Tracker for V4L camera " <<
+                Config.m_sDevice << " width=" << Config.m_Size.x << " height=" <<
+                Config.m_Size.y << " channel=" << Config.m_Channel);
+        
+        // is there a need to get a configurable pixel format for tracker?
+        pCamera = CameraPtr(new V4LCamera(Config.m_sDevice, Config.m_Channel, 
+                Config.m_Size, "MONO8", false));
+#else
+        AVG_TRACE(Logger::ERROR, "Video4Linux camera tracker requested, but " 
+                "Video4Linux support not compiled in.");
+        exit(1);
+#endif
+    } else {
+        AVG_TRACE(Logger::CONFIG, "Adding a Tracker for FW camera " << 
+                Config.m_sDevice << " width=" << Config.m_Size.x << 
+                " height=" << Config.m_Size.y);
+        pCamera = CameraPtr(new FWCamera(Config.m_sDevice, Config.m_Size, 
+                "MONO8", Config.m_FPS, false));
+    }
+    
+    m_pTracker = new TrackerEventSource(pCamera, Config, 
+            IntPoint(m_DP.m_Width, m_DP.m_Height), true);
+    m_pEventDispatcher->addSource(m_pTracker);
+    return m_pTracker;
+}
+
 int Player::setInterval(int time, PyObject * pyfunc)
 {
-    Timeout *t = new Timeout(time, pyfunc, true);
+    Timeout *t = new Timeout(time, pyfunc, true, getFrameTime());
     if (m_bInHandleTimers) {
         m_NewTimeouts.push_back(t);
     } else {
@@ -308,7 +371,7 @@ int Player::setInterval(int time, PyObject * pyfunc)
 
 int Player::setTimeout(int time, PyObject * pyfunc)
 {
-    Timeout *t = new Timeout(time, pyfunc, false);
+    Timeout *t = new Timeout(time, pyfunc, false, getFrameTime());
     if (m_bInHandleTimers) {
         m_NewTimeouts.push_back(t);
     } else {
@@ -317,6 +380,11 @@ int Player::setTimeout(int time, PyObject * pyfunc)
     return t->GetID();
 }
 
+int Player::setOnFrameHandler(PyObject * pyfunc) 
+{
+    return setInterval(0, pyfunc);
+}
+
 bool Player::clearInterval(int id)
 {
     vector<Timeout*>::iterator it;
@@ -330,17 +398,19 @@ bool Player::clearInterval(int id)
             return true;
         }
     }
+    for (it=m_NewTimeouts.begin(); it!=m_NewTimeouts.end(); it++) {
+        if (id == (*it)->GetID()) {
+            delete *it;
+            m_NewTimeouts.erase(it);
+            return true;
+        }
+    }
     return false;
 }
 
-const Event& Player::getCurEvent() const
-{
-    return *m_pCurEvent;
-}
-
 const MouseEvent& Player::getMouseState() const
 {
-    return m_EventDispatcher.getLastMouseEvent();
+    return m_pEventDispatcher->getLastMouseEvent();
 }
 
 Bitmap * Player::screenshot()
@@ -358,21 +428,23 @@ void Player::showCursor(bool bShow)
     }
 }
 
-void Player::setEventCapture(NodeWeakPtr pNode) {
-    if (!m_pEventCaptureNode.expired()) {
+void Player::setEventCapture(NodeWeakPtr pNode, int cursorID) {
+    std::map<int, NodeWeakPtr>::iterator it = m_pEventCaptureNode.find(cursorID);
+    if (it!=m_pEventCaptureNode.end()&&!it->second.expired()) {
         throw Exception(AVG_ERR_INVALID_CAPTURE,
-                "setEventCapture called, but mouse already captured.");
+                "setEventCapture called, but cursor already captured.");
     } else {
-        m_pEventCaptureNode = pNode;
+        m_pEventCaptureNode[cursorID] = pNode;
     }
 }
 
-void Player::releaseEventCapture(NodeWeakPtr pNode) {
-    if (m_pEventCaptureNode.expired()) {
+void Player::releaseEventCapture(NodeWeakPtr pNode, int cursorID) {
+    std::map<int, NodeWeakPtr>::iterator it = m_pEventCaptureNode.find(cursorID);
+    if(it==m_pEventCaptureNode.end()||(it->second.expired()) ) {
         throw Exception(AVG_ERR_INVALID_CAPTURE,
-                "releaseEventCapture called, but mouse not captured.");
+                "releaseEventCapture called, but cursor not captured.");
     } else {
-        m_pEventCaptureNode = NodeWeakPtr();
+        m_pEventCaptureNode.erase(cursorID);
     }
 
 }
@@ -409,9 +481,9 @@ void Player::removeNodeID(const std::string& id)
         if (it != m_IDMap.end()) {
             m_IDMap.erase(it);
         } else {
-            AVG_TRACE(Logger::ERROR, "removeNodeID("+id+") failed.");
+            AVG_TRACE(Logger::ERROR, "removeNodeID(\""+id+"\") failed.");
             exit(1);
-        }
+      }
     }
 }
 
@@ -451,17 +523,26 @@ void Player::doFrame ()
 {
     {
         ScopeTimer Timer(MainProfilingZone);
+        if (m_bFakeFPS) {
+            m_NumFrames++;
+            m_FrameTime = (long long)((m_NumFrames*1000.0)/m_FakeFPS);
+        } else {
+            m_FrameTime = m_pDisplayEngine->getDisplayTime();
+//            m_FrameTime = TimeSource::get()->getCurrentMillisecs()-m_PlayStartTime;
+        }
         {
             ScopeTimer Timer(TimersProfilingZone);
             handleTimers();
         }
         {
             ScopeTimer Timer(EventsProfilingZone);
-            m_EventDispatcher.dispatch();
+            m_pEventDispatcher->dispatch();
         }
         if (!m_bStopping) {
+            Py_BEGIN_ALLOW_THREADS;
             ScopeTimer Timer(RenderProfilingZone);
             m_pDisplayEngine->render(m_pRootNode, false);
+            Py_END_ALLOW_THREADS;
         }
         {
             ScopeTimer Timer(ListenerProfilingZone);
@@ -471,7 +552,7 @@ void Player::doFrame ()
         }
     }
     if (m_pDisplayEngine->wasFrameLate()) {
-        Profiler::get().dumpFrame();
+        ThreadProfiler::get()->dumpFrame();
     }
     
 /*
@@ -482,8 +563,8 @@ void Player::doFrame ()
                 FrameTime-TargetTime << " ms.");
         Profiler::get().dumpFrame();
     }
-*/    
-    Profiler::get().reset();
+*/
+    ThreadProfiler::get()->reset();
 }
 
 double Player::getFramerate ()
@@ -517,8 +598,6 @@ void Player::initConfig() {
     // Get data from config files.
     ConfigMgr* pMgr = ConfigMgr::get();
     
-    m_sDisplaySubsystem = *pMgr->getOption("scr", "subsys");
-   
     m_DP.m_BPP = atoi(pMgr->getOption("scr", "bpp")->c_str());
     if (m_DP.m_BPP != 15 && m_DP.m_BPP != 16 && m_DP.m_BPP != 24 && m_DP.m_BPP != 32) {
         AVG_TRACE(Logger::ERROR, 
@@ -543,110 +622,97 @@ void Player::initConfig() {
         exit(-1);
     }
 
-    if (m_sDisplaySubsystem == "DFB") {
-        if  (pMgr->getOption("scr", "usepow2textures") != 0 || 
-             pMgr->getOption("scr", "ycbcrmode") != 0 ||
-             pMgr->getOption("scr", "usergborder") != 0 ||
-             pMgr->getOption("scr", "usepixelbuffers") != 0) 
-        {
-            AVG_TRACE(Logger::WARNING, 
-                    "avgrc: usepow2textures, ycbcrmode, usergborder and usepixelbuffers are unsupported in the DirectFB backend. Ignoring.");
-        }
-    } else {
-        m_bUsePOW2Textures = pMgr->getBoolOption("scr", "usepow2textures", false);
-        
-        const string * psYCbCrMode =pMgr->getOption("scr", "ycbcrmode");
-        if (psYCbCrMode == 0 || *psYCbCrMode == "shader") {
-            m_YCbCrMode = DisplayEngine::OGL_SHADER;
-        } else if (*psYCbCrMode == "mesa") {
-            m_YCbCrMode = DisplayEngine::OGL_MESA;
-        } else if (*psYCbCrMode == "apple") {
-            m_YCbCrMode = DisplayEngine::OGL_APPLE;
-        } else if (*psYCbCrMode == "none") {
-            m_YCbCrMode = DisplayEngine::NONE;
-        } else {
-            AVG_TRACE(Logger::ERROR, 
-                    "avgrc: ycbcrmode must be shader, mesa, apple or none. Current value is " 
-                    << *psYCbCrMode << ". Aborting." );
-            exit(-1);
-        }
+    m_bUsePOW2Textures = pMgr->getBoolOption("scr", "usepow2textures", false);
 
-        m_bUseRGBOrder = pMgr->getBoolOption("scr", "usergborder", false);
-        m_bUsePixelBuffers = pMgr->getBoolOption("scr", "usepixelbuffers", true);
-        m_MultiSampleSamples = pMgr->getIntOption("scr", "multisamplesamples", 1);
-        pMgr->getGammaOption("scr", "gamma", m_DP.m_Gamma);
+    const string * psYCbCrMode =pMgr->getOption("scr", "ycbcrmode");
+    if (psYCbCrMode == 0 || *psYCbCrMode == "shader") {
+        m_YCbCrMode = OGL_SHADER;
+    } else if (*psYCbCrMode == "mesa") {
+        m_YCbCrMode = OGL_MESA;
+    } else if (*psYCbCrMode == "apple") {
+        m_YCbCrMode = OGL_APPLE;
+    } else if (*psYCbCrMode == "none") {
+        m_YCbCrMode = OGL_NONE;
+    } else {
+        AVG_TRACE(Logger::ERROR, 
+                "avgrc: ycbcrmode must be shader, mesa, apple or none. Current value is " 
+                << *psYCbCrMode << ". Aborting." );
+        exit(-1);
     }
+
+    const string * psVSyncMode =pMgr->getOption("scr", "vsyncmode");
+    if (psVSyncMode == 0 || *psVSyncMode == "auto") {
+        m_VSyncMode = VSYNC_AUTO;
+    } else if (*psVSyncMode == "ogl") {
+        m_VSyncMode = VSYNC_OGL;
+    } else if (*psVSyncMode == "dri") {
+        m_VSyncMode = VSYNC_DRI;
+    } else if (*psVSyncMode == "none") {
+        m_VSyncMode = VSYNC_NONE;
+    } else {
+        AVG_TRACE(Logger::ERROR, 
+                "avgrc: vsyncmode must be auto, ogl, dri or none. Current value is " 
+                << *psVSyncMode << ". Aborting." );
+        exit(-1);
+    }
+
+    m_bUsePixelBuffers = pMgr->getBoolOption("scr", "usepixelbuffers", true);
+    m_MultiSampleSamples = pMgr->getIntOption("scr", "multisamplesamples", 1);
+    pMgr->getGammaOption("scr", "gamma", m_DP.m_Gamma);
 }
 
 void Player::initGraphics()
 {
     // Init display configuration.
-    AVG_TRACE(Logger::CONFIG, "Display subsystem: " << 
-            m_sDisplaySubsystem);
     AVG_TRACE(Logger::CONFIG, "Display bpp: " << m_DP.m_BPP);
-    AVG_TRACE(Logger::CONFIG, "Display fullscreen: "
-            << (m_DP.m_bFullscreen?"true":"false"));
 
     if (!m_pDisplayEngine) {
-        if (m_sDisplaySubsystem == "DFB") {
-#ifdef AVG_ENABLE_DFB
-            m_pDisplayEngine = new DFBDisplayEngine ();
-            m_pEventSource =
-                dynamic_cast<DFBDisplayEngine *>(m_pDisplayEngine);
-#else
-            AVG_TRACE(Logger::ERROR,
-                    "Display subsystem set to DFB but no DFB support compiled."
-                    << " Aborting.");
-            exit(-1);
-#endif
-        } else if (m_sDisplaySubsystem == "OGL") {
-#ifdef AVG_ENABLE_GL
-            AVG_TRACE(Logger::CONFIG, "Requested OGL configuration: ");
-            AVG_TRACE(Logger::CONFIG, "  POW2 textures: " 
-                    << (m_bUsePOW2Textures?"true":"false"));
-            string sMode;
-            switch (m_YCbCrMode) {
-                case DisplayEngine::NONE:
-                    AVG_TRACE(Logger::CONFIG, "  No YCbCr texture support.");
-                    break;
-                case DisplayEngine::OGL_MESA:
-                    AVG_TRACE(Logger::CONFIG, "  Mesa YCbCr texture support.");
-                    break;
-                case DisplayEngine::OGL_APPLE:
-                    AVG_TRACE(Logger::CONFIG, "  Apple YCbCr texture support.");
-                    break;
-                case DisplayEngine::OGL_SHADER:
-                    AVG_TRACE(Logger::CONFIG, "  Fragment shader YCbCr texture support.");
-                    break;
-            }
-            AVG_TRACE(Logger::CONFIG, "  RGB order: " 
-                    << (m_bUseRGBOrder?"true":"false"));
-            AVG_TRACE(Logger::CONFIG, "  Use pixel buffers: " 
-                    << (m_bUsePixelBuffers?"true":"false"));
-            AVG_TRACE(Logger::CONFIG, "  Multisample samples: " 
-                    << m_MultiSampleSamples);
-
-            m_pDisplayEngine = new SDLDisplayEngine ();
-            m_pEventSource = 
-                dynamic_cast<SDLDisplayEngine *>(m_pDisplayEngine);
-#else
-            AVG_TRACE(Logger::ERROR,
-                    "Display subsystem set to GL but no GL support compiled."
-                    << " Aborting.");
-            exit(-1);
-#endif
-        } else {
-            AVG_TRACE(Logger::ERROR, 
-                    "Display subsystem set to unknown value " << 
-                    m_sDisplaySubsystem << ". Aborting.");
-            exit(-1);
+        AVG_TRACE(Logger::CONFIG, "Requested OpenGL configuration: ");
+        AVG_TRACE(Logger::CONFIG, "  POW2 textures: " 
+                << (m_bUsePOW2Textures?"true":"false"));
+        string sMode;
+        switch (m_YCbCrMode) {
+            case OGL_NONE:
+                AVG_TRACE(Logger::CONFIG, "  No YCbCr texture support.");
+                break;
+            case OGL_MESA:
+                AVG_TRACE(Logger::CONFIG, "  Mesa YCbCr texture support.");
+                break;
+            case OGL_APPLE:
+                AVG_TRACE(Logger::CONFIG, "  Apple YCbCr texture support.");
+                break;
+            case OGL_SHADER:
+                AVG_TRACE(Logger::CONFIG, "  Fragment shader YCbCr texture support.");
+                break;
         }
+        AVG_TRACE(Logger::CONFIG, "  Use pixel buffers: " 
+                << (m_bUsePixelBuffers?"true":"false"));
+        AVG_TRACE(Logger::CONFIG, "  Multisample samples: " 
+                << m_MultiSampleSamples);
+        switch (m_VSyncMode) {
+            case VSYNC_AUTO:
+                AVG_TRACE(Logger::CONFIG, "  Auto vsync");
+                break;
+            case VSYNC_OGL:
+                AVG_TRACE(Logger::CONFIG, "  OpenGL vsync");
+                break;
+            case VSYNC_DRI:
+                AVG_TRACE(Logger::CONFIG, "  DRI vsync");
+                break;
+            case VSYNC_NONE:
+                AVG_TRACE(Logger::CONFIG, "  No vsync");
+                break;
+        }
+
+        m_pDisplayEngine = new SDLDisplayEngine ();
+        m_pEventSource = 
+            dynamic_cast<SDLDisplayEngine *>(m_pDisplayEngine);
     }
     SDLDisplayEngine * pSDLDisplayEngine = 
             dynamic_cast<SDLDisplayEngine*>(m_pDisplayEngine);
     if (pSDLDisplayEngine) {
         pSDLDisplayEngine->setOGLOptions(m_bUsePOW2Textures, m_YCbCrMode, 
-                m_bUseRGBOrder, m_bUsePixelBuffers, m_MultiSampleSamples);
+                m_bUsePixelBuffers, m_MultiSampleSamples, m_VSyncMode);
     }
     m_pDisplayEngine->init(m_DP);
 }
@@ -717,7 +783,7 @@ NodePtr Player::createNodeFromXml (const xmlDocPtr xmlDoc,
     } else if (!strcmp (nodeType, "video")) {
         curNode = NodePtr(new Video(xmlNode, this));
     } else if (!strcmp (nodeType, "camera")) {
-        curNode = NodePtr(new Camera(xmlNode, this));
+        curNode = NodePtr(new CameraNode(xmlNode, this));
     }
     else if (!strcmp (nodeType, "panoimage")) {
         curNode = NodePtr(new PanoImage(xmlNode, this));
@@ -739,7 +805,7 @@ NodePtr Player::createNodeFromXml (const xmlDocPtr xmlDoc,
             NodePtr curChild = createNodeFromXml (xmlDoc, curXmlChild, 
                     curDivNode);
             if (curChild) {
-                curDivNode->addChild(curChild);
+                curDivNode->appendChild(curChild);
             }
             curXmlChild = curXmlChild->next;
         }
@@ -754,9 +820,9 @@ void Player::handleTimers()
     vector<Timeout *> IntervalsFired;
     
     it = m_PendingTimeouts.begin();
-    while (it != m_PendingTimeouts.end() && (*it)->IsReady() && !m_bStopping)
+    while (it != m_PendingTimeouts.end() && (*it)->IsReady(getFrameTime()) && !m_bStopping)
     {
-        (*it)->Fire();
+        (*it)->Fire(getFrameTime());
         if (!m_bCurrentTimeoutDeleted) {
             if ((*it)->IsInterval()) {
                 Timeout* pTempTimeout = *it;
@@ -780,67 +846,68 @@ void Player::handleTimers()
 
 bool Player::handleEvent(Event * pEvent)
 {
-    m_pCurEvent = pEvent;
-    switch (pEvent->getType()) {
-        case Event::MOUSEMOTION:
-        case Event::MOUSEBUTTONUP:
-        case Event::MOUSEBUTTONDOWN:
-        case Event::MOUSEOVER:
-        case Event::MOUSEOUT:
-            {
-                MouseEvent * pMouseEvent = dynamic_cast<MouseEvent*>(pEvent);
-                DPoint pos(pMouseEvent->getXPosition(), 
-                        pMouseEvent->getYPosition());
-                NodePtr pNode;
-                if (m_pEventCaptureNode.expired()) {
-                    if (pEvent->getType() != Event::MOUSEOVER &&
-                            pEvent->getType() != Event::MOUSEOUT)
-                    {
-                        pNode = m_pRootNode->getElementByPos(pos);
-                    } else {
-                        pNode = pMouseEvent->getElement();
-                    }
-                } else {
-                    pNode = m_pEventCaptureNode.lock();
-                } 
-                if (pNode != m_pLastMouseNode && 
-                        pEvent->getType() != Event::MOUSEOVER &&
-                        pEvent->getType() != Event::MOUSEOUT)
-                {
-                    if (m_pLastMouseNode && m_pLastMouseNode->getSensitive()) {
-                        sendMouseOver(pMouseEvent, Event::MOUSEOUT, 
-                                m_pLastMouseNode);
-                    }
-                    if (pNode && pNode->getSensitive()) {
-                        sendMouseOver(pMouseEvent, Event::MOUSEOVER, pNode);
-                    }
-                    m_pLastMouseNode = pNode;
-                }
-                if (pNode && pNode->getSensitive()) {
-                    pNode->handleMouseEvent(pMouseEvent);
-                }
+    assert(pEvent); 
+    if(CursorEvent * pCursorEvent = dynamic_cast<CursorEvent*>(pEvent)) {
+        DPoint pos(pCursorEvent->getXPosition(), 
+                pCursorEvent->getYPosition());
+        int cursorID = pCursorEvent->getCursorID();
+        NodePtr pNode;
+        if (m_pEventCaptureNode.find(cursorID) != m_pEventCaptureNode.end()) {
+            NodeWeakPtr pEventCaptureNode = m_pEventCaptureNode[cursorID];
+            if (pEventCaptureNode.expired()) {
+                m_pEventCaptureNode.erase(cursorID);
+            } else {
+                pNode = m_pEventCaptureNode[cursorID].lock();
             }
-            break;
-        case Event::KEYDOWN:
-        case Event::KEYUP:
+        } 
+        if (!pNode) {
+            if (pEvent->getType() == Event::CURSOROVER ||
+                    pEvent->getType() == Event::CURSOROUT)
             {
-                KeyEvent * pKeyEvent = dynamic_cast<KeyEvent*>(pEvent);
-                m_pRootNode->handleKeyEvent(pKeyEvent);
-                if (pEvent->getType() == Event::KEYDOWN &&
-                    pKeyEvent->getKeyCode() == 27) 
-                {
-                    m_bStopping = true;
-                }
+                pNode = pCursorEvent->getElement();
+            } else {
+                pNode = m_pRootNode->getElementByPos(pos);
             }
-            break;
-        case Event::QUIT:
+        }
+        if (pNode != m_pLastMouseNode[cursorID] && 
+                pEvent->getType() != Event::CURSOROVER &&
+                pEvent->getType() != Event::CURSOROUT)
+        {
+            if (m_pLastMouseNode[cursorID] && m_pLastMouseNode[cursorID]->getSensitive()) {
+                sendOver(pCursorEvent, Event::CURSOROUT, 
+                        m_pLastMouseNode[cursorID]);
+            }
+            if (pNode && pNode->getSensitive()) {
+                sendOver(pCursorEvent, Event::CURSOROVER, pNode);
+            }
+            m_pLastMouseNode[cursorID] = pNode;
+        }
+        if (pCursorEvent->getType() == Event::CURSORUP && 
+            pCursorEvent->getSource() != CursorEvent::MOUSE) 
+        {
+            m_pLastMouseNode.erase(cursorID);
+        }
+        if (pNode && pNode->getSensitive()) {
+            pNode->handleEvent(pCursorEvent);
+        }
+    } else if ( KeyEvent * pKeyEvent = dynamic_cast<KeyEvent*>(pEvent)){
+        m_pRootNode->handleEvent(pKeyEvent);
+        if (pEvent->getType() == Event::KEYDOWN &&
+            pKeyEvent->getKeyCode() == 27) 
+        {
             m_bStopping = true;
-            break;
-        default:
-            AVG_TRACE(Logger::ERROR, "Unknown event type in Player::handleEvent.");
-            break;
-    }
+        }
+    }else {
+        switch(pEvent->getType()){
+            case Event::QUIT:
+                m_bStopping = true;
+                break;
+            default:
+                AVG_TRACE(Logger::ERROR, "Unknown event type in Player::handleEvent.");
+                break;
+        }
     // Don't pass on any events.
+    }
     return true; 
 }
 
@@ -849,18 +916,17 @@ DisplayEngine * Player::getDisplayEngine() const
     return m_pDisplayEngine;
 }
 
-void Player::sendMouseOver(MouseEvent * pOtherEvent, Event::Type Type, 
+void Player::useFakeCamera(bool bFake)
+{
+    m_bUseFakeCamera = bFake;
+}
+
+void Player::sendOver(CursorEvent * pOtherEvent, Event::Type Type, 
                 NodePtr pNode)
 {
-    MouseEvent * pNewEvent = new MouseEvent(Type,
-            pOtherEvent->getLeftButtonState(),
-            pOtherEvent->getMiddleButtonState(),
-            pOtherEvent->getRightButtonState(),
-            pOtherEvent->getXPosition(),
-            pOtherEvent->getYPosition(),
-            pOtherEvent->getButton());
+    Event * pNewEvent = pOtherEvent->cloneAs(Type);
     pNewEvent->setElement(pNode);
-    m_EventDispatcher.sendEvent(pNewEvent);
+    m_pEventDispatcher->sendEvent(pNewEvent);
 }
 
 void Player::cleanup() 
@@ -872,14 +938,17 @@ void Player::cleanup()
     }
     m_PendingTimeouts.clear();
     Profiler::get().dumpStatistics();
+    m_pRootNode->disconnect();
+    m_pRootNode = AVGNodePtr();
     if (m_pDisplayEngine) {
         m_pDisplayEngine->deinitRender();
         m_pDisplayEngine->teardown();
     }
-    m_pRootNode = AVGNodePtr();
-    m_pLastMouseNode = NodePtr();
+    m_pLastMouseNode.clear();
     m_IDMap.clear();
+    m_pEventDispatcher = EventDispatcherPtr();
     initConfig();
+    m_FrameTime = 0;
 }
 
 int Player::addTimeout(Timeout* pTimeout)
