@@ -44,8 +44,7 @@
 namespace avg {
 
 class Event;
-class MouseEvent;
-class DisplayEngine;
+class TrackerEventSource;
 
 class Player : IEventSink
 {
@@ -53,32 +52,38 @@ class Player : IEventSink
         Player();
         virtual ~Player();
 
-        enum DisplayEngineType{DFB, OGL};
-        void setDisplayEngine(DisplayEngineType engine);
         void setResolution(bool bFullscreen, 
                 int width=0, int height=0, int bpp=0);
-        void setOGLOptions(bool bUsePOW2Textures, DisplayEngine::YCbCrMode DesiredYCbCrMode, 
-                bool bUseRGBOrder, bool bUsePixelBuffers, int MultiSampleSamples);
+        void setOGLOptions(bool bUsePOW2Textures, YCbCrMode DesiredYCbCrMode, 
+                bool bUsePixelBuffers, int MultiSampleSamples);
         void loadFile (const std::string& fileName);
         void play();
         void stop();
         bool isPlaying();
         void setFramerate(double rate);
         bool setVBlankFramerate(int rate);
+        double getEffectiveFramerate();
+        unsigned getMemUsed();
         TestHelper * getTestHelper();
+        void setFakeFPS(double fps);
+        long long getFrameTime();
 
         NodePtr createNodeFromXmlString (const std::string& sXML);
+        TrackerEventSource * addTracker();
         int setInterval(int time, PyObject * pyfunc);
         int setTimeout(int time, PyObject * pyfunc);
+        int setOnFrameHandler(PyObject * pyfunc);
         bool clearInterval(int id);
+
         const Event& getCurEvent() const;
         const MouseEvent& getMouseState() const;
         Bitmap * screenshot();
         void showCursor(bool bShow);
-        void setEventCapture(NodeWeakPtr pNode);
-        void releaseEventCapture(NodeWeakPtr pNode);
+        void setEventCapture(NodeWeakPtr pNode, int cursorID=MOUSECURSORID);
+        void releaseEventCapture(NodeWeakPtr pNode, int cursorID=MOUSECURSORID);
 
         NodePtr getElementByID(const std::string& id);
+        void registerNode(NodePtr pNode);
         void addNodeID(NodePtr pNode);
         void removeNodeID(const std::string& id);
         AVGNodePtr getRootNode();
@@ -88,6 +93,7 @@ class Player : IEventSink
         void setGamma(double Red, double Green, double Blue);
         virtual bool handleEvent(Event * pEvent);
         DisplayEngine * getDisplayEngine() const;
+        void useFakeCamera(bool bFake);
 
         void registerFrameListener(IFrameListener* pListener);
         void unregisterFrameListener(IFrameListener* pListener);
@@ -96,16 +102,15 @@ class Player : IEventSink
     private:
         void initConfig();
         void initGraphics();
-        void registerNode(NodePtr pNode);
 
         NodePtr createNodeFromXml(const xmlDocPtr xmlDoc, 
                 const xmlNodePtr xmlNode, DivNodeWeakPtr pParent);
 
         void render (bool bRenderEverything);
-        void sendMouseOver(MouseEvent * pOtherEvent, Event::Type Type, 
+        void sendOver(CursorEvent * pOtherEvent, Event::Type Type, 
                 NodePtr pNode);
         void cleanup();
-	
+
         AVGNodePtr m_pRootNode;
         DisplayEngine * m_pDisplayEngine;
         IEventSource * m_pEventSource;
@@ -116,6 +121,8 @@ class Player : IEventSink
         typedef std::map<std::string, NodePtr> NodeIDMap;
         NodeIDMap m_IDMap;
 
+        TrackerEventSource * m_pTracker;
+
         int addTimeout(Timeout* pTimeout);
         void removeTimeout(Timeout* pTimeout);
         void handleTimers();
@@ -125,24 +132,29 @@ class Player : IEventSink
         std::vector<Timeout *> m_PendingTimeouts;
         std::vector<Timeout *> m_NewTimeouts; // Timeouts to be added this frame.
 
-        EventDispatcher m_EventDispatcher;
+        EventDispatcherPtr m_pEventDispatcher;
         DebugEventSink  m_EventDumper;
-        Event * m_pCurEvent;
-        NodePtr m_pLastMouseNode;
-        NodeWeakPtr m_pEventCaptureNode;
+        std::map<int, NodePtr> m_pLastMouseNode;
+        std::map<int, NodeWeakPtr> m_pEventCaptureNode;
 
         // Configuration variables.
-        std::string m_sDisplaySubsystem;
         DisplayParams m_DP;
-        
         bool m_bUsePOW2Textures;
-        DisplayEngine::YCbCrMode m_YCbCrMode;
-        bool m_bUseRGBOrder;
+        YCbCrMode m_YCbCrMode;
         bool m_bUsePixelBuffers;
         int m_MultiSampleSamples;
+        bool m_bUseFakeCamera;
+        VSyncMode m_VSyncMode;
 
         bool m_bIsPlaying;
 
+        // Time calculation
+        bool m_bFakeFPS;
+        double m_FakeFPS;
+        long long m_FrameTime;
+        long long m_PlayStartTime;
+        long long m_NumFrames;
+
         std::vector<IFrameListener*> m_Listeners;
         xmlDtdPtr m_dtd;
 };
