@@ -19,13 +19,8 @@
 //  Current versions can be found at www.libavg.de
 //
 
-#include "../avgconfig.h"
-
 #include "RasterNode.h"
 
-#ifdef AVG_ENABLE_GL
-#include "OGLSurface.h"
-#endif
 #include "MathHelper.h"
 #include "DisplayEngine.h"
 
@@ -36,15 +31,6 @@ using namespace std;
 
 namespace avg {
 
-RasterNode::RasterNode ()
-    : m_pSurface(0),
-      m_Angle(0),
-      m_Pivot(-32767, -32767),
-      m_MaxTileSize(IntPoint(-1,-1)),
-      m_sBlendMode("blend")
-{
-}
-
 RasterNode::RasterNode (const xmlNodePtr xmlNode, Player * pPlayer)
     : Node(xmlNode, pPlayer),
       m_pSurface(0),
@@ -65,6 +51,7 @@ RasterNode::~RasterNode()
 {
     if (m_pSurface) {
         delete m_pSurface;
+        m_pSurface = 0;
     }
 }
 
@@ -75,23 +62,9 @@ void RasterNode::setDisplayEngine(DisplayEngine * pEngine)
     m_bHasCustomPivot = ((m_Pivot.x != -32767) && (m_Pivot.y != -32767));
 
     if (m_MaxTileSize != IntPoint(-1, -1)) {
-#ifdef AVG_ENABLE_GL        
         OGLSurface * pOGLSurface = 
             dynamic_cast<OGLSurface*>(getSurface());
-        if (!pOGLSurface) {
-            AVG_TRACE(Logger::WARNING, 
-                    "Node "+getID()+": "
-                    "Custom tile sizes are only allowed when "
-                    "the display engine is OpenGL. Ignoring.");
-        } else {
-            pOGLSurface->setMaxTileSize(m_MaxTileSize);
-        }
-#else
-            AVG_TRACE(Logger::WARNING, 
-                    "Node "+getID()+": "
-                    "Custom tile sizes are only allowed when "
-                    "the display engine is OpenGL. Ignoring.");
-#endif        
+        pOGLSurface->setMaxTileSize(m_MaxTileSize);
     }
     setBlendModeStr(m_sBlendMode);
 }
@@ -105,52 +78,22 @@ void RasterNode::disconnect()
     Node::disconnect();
 }
 
-int RasterNode::getNumVerticesX()
+VertexGrid RasterNode::getOrigVertexCoords()
 {
-#ifdef AVG_ENABLE_GL
     OGLSurface * pOGLSurface = getOGLSurface();
-    return pOGLSurface->getNumVerticesX(); 
-#else
-    return 1;
-#endif
+    return pOGLSurface->getOrigVertexCoords();
 }
 
-int RasterNode::getNumVerticesY()
+VertexGrid RasterNode::getWarpedVertexCoords() 
 {
-#ifdef AVG_ENABLE_GL
     OGLSurface * pOGLSurface = getOGLSurface();
-    return pOGLSurface->getNumVerticesY(); 
-#else
-    return 1;
-#endif
+    return pOGLSurface->getWarpedVertexCoords();
 }
 
-DPoint RasterNode::getOrigVertexCoord(int x, int y)
+void RasterNode::setWarpedVertexCoords(const VertexGrid& Grid)
 {
-#ifdef AVG_ENABLE_GL
     OGLSurface * pOGLSurface = getOGLSurface();
-    return pOGLSurface->getOrigVertexCoord(x, y);
-#else
-    return DPoint(0,0);
-#endif
-}
-
-DPoint RasterNode::getWarpedVertexCoord(int x, int y) 
-{
-#ifdef AVG_ENABLE_GL
-    OGLSurface * pOGLSurface = getOGLSurface();
-    return pOGLSurface->getWarpedVertexCoord(x, y);
-#else
-    return DPoint(0,0);
-#endif
-}
-
-void RasterNode::setWarpedVertexCoord(int x, int y, const DPoint& Vertex)
-{
-#ifdef AVG_ENABLE_GL
-    OGLSurface * pOGLSurface = getOGLSurface();
-    pOGLSurface->setWarpedVertexCoord(x, y, Vertex);
-#endif
+    pOGLSurface->setWarpedVertexCoords(Grid);
 }
 
 double RasterNode::getAngle() const
@@ -161,7 +104,6 @@ double RasterNode::getAngle() const
 void RasterNode::setAngle(double Angle)
 {
     m_Angle = fmod(Angle, 2*PI);
-    invalidate();
 }
 
 double RasterNode::getPivotX() const
@@ -272,20 +214,11 @@ DPoint RasterNode::getPivot()
     }
 }
 
-#ifdef AVG_ENABLE_GL
 OGLSurface * RasterNode::getOGLSurface()
 {
     OGLSurface * pOGLSurface = dynamic_cast<OGLSurface *>(getSurface());
-    if (pOGLSurface) {
-        return pOGLSurface; 
-    } else {
-        AVG_TRACE(Logger::ERROR, 
-                "OpenGL display engine needed for node " << getID() <<
-                ". Aborting.");
-        exit(-1);
-    }
+    return pOGLSurface; 
 }
-#endif
 
 DisplayEngine::BlendMode RasterNode::getBlendMode() const
 {
