@@ -24,12 +24,13 @@
 
 #include "Node.h"
 #include "DisplayEngine.h"
-#include "../graphics/Point.h"
+#include "OGLSurface.h"
+
+#include "../avgconfigwrapper.h"
+#include "../base/Point.h"
 
 #include <string>
 
-#include "../avgconfig.h"
-
 namespace avg {
 
 class RasterNode: public Node
@@ -40,11 +41,9 @@ class RasterNode: public Node
         virtual void disconnect();
         
         // Warping support.
-        int getNumVerticesX();
-        int getNumVerticesY();
-        DPoint getOrigVertexCoord(int x, int y);
-        DPoint getWarpedVertexCoord(int x, int y);
-        void setWarpedVertexCoord(int x, int y, const DPoint& Vertex);
+        VertexGrid getOrigVertexCoords();
+        VertexGrid getWarpedVertexCoords();
+        void setWarpedVertexCoords(const VertexGrid& Grid);
 
         double getAngle() const;
         void setAngle(double Angle);
@@ -65,9 +64,7 @@ class RasterNode: public Node
         
         const std::string& getBlendModeStr() const;
         void setBlendModeStr(const std::string& sBlendMode);
-#ifdef AVG_ENABLE_GL
         OGLSurface * getOGLSurface();
-#endif
         DisplayEngine::BlendMode getBlendMode() const;
         virtual std::string getTypeStr ();
         NodePtr getElementByPos (const DPoint & pos);
@@ -75,7 +72,6 @@ class RasterNode: public Node
         Bitmap* getBitmap();
         
     protected:
-        RasterNode ();
         RasterNode (const xmlNodePtr xmlNode, Player * pPlayer);
         DPoint getPivot();
         ISurface * getSurface();
