@@ -76,7 +76,14 @@ using namespace std;
 namespace avg {
 
 #ifdef linux
-#define DRM_VBLANK_RELATIVE 0x1;
+
+typedef enum {
+    DRM_VBLANK_ABSOLUTE = 0x0,  /**< Wait for specific vblank sequence number */
+    DRM_VBLANK_RELATIVE = 0x1,  /**< Wait for given number of vblanks */
+    DRM_VBLANK_NEXTONMISS = 0x10000000, /**< If missed, wait for next vblank */
+    DRM_VBLANK_SECONDARY = 0x20000000,  /**< Secondary display controller */
+    DRM_VBLANK_SIGNAL   = 0x40000000    /* Send signal instead of blocking */
+} drmVBlankSeqType;
 
 struct drm_wait_vblank_request {
     int type;
@@ -103,14 +110,25 @@ typedef union drm_wait_vblank {
 
 static int drmWaitVBlank(int fd, drm_wait_vblank_t *vbl)
 {
+    static bool bUseSecondaryPipe = false;
     int ret;
     int rc;
 
     do {
        ret = ioctl(fd, DRM_IOCTL_WAIT_VBLANK, vbl);
+       if (bUseSecondaryPipe) {
+           vbl->request.type |= DRM_VBLANK_SECONDARY;
+       }
        vbl->request.type &= ~DRM_VBLANK_RELATIVE;
        rc = errno;
     } while (ret && rc == EINTR);
+    if (!bUseSecondaryPipe && rc == EBUSY) {
+        // rc 16 == device or resource busy
+        AVG_TRACE(Logger::CONFIG, 
+                "Using secondary pipe for vblank.");
+        bUseSecondaryPipe = true;
+        rc = 0;
+    }
 
     return rc;
 }
@@ -142,8 +160,10 @@ void dumpSDLGLParams() {
 SDLDisplayEngine::SDLDisplayEngine()
     : m_bEnableCrop(false),
       m_pScreen(0),
+      m_VBMethod(VB_NONE),
       m_VBMod(0),
       m_dri_fd(0),
+      m_bMouseOverApp(true),
       m_TextureMode(0),
       m_MaxTexSize(0),
       m_bCheckedMemoryMode(false)
@@ -268,7 +288,6 @@ void SDLDisplayEngine::init(const DisplayParams& DP)
     initInput();
     // SDL sets up a signal handler we really don't want.
     signal(SIGSEGV, SIG_DFL);
-    
     logConfig();
     
 //    dumpSDLGLParams();
@@ -305,33 +324,53 @@ void SDLDisplayEngine::setGamma(double Red, double Green, double Blue)
 
 void SDLDisplayEngine::logConfig() 
 {
-    AVG_TRACE(Logger::CONFIG, "OpenGL version: " << glGetString(GL_VERSION));
-    AVG_TRACE(Logger::CONFIG, "OpenGL vendor: " << glGetString(GL_VENDOR));
-    AVG_TRACE(Logger::CONFIG, "OpenGL renderer: " << glGetString(GL_RENDERER));
+    AVG_TRACE(Logger::CONFIG, "OpenGL configuration: ");
+    AVG_TRACE(Logger::CONFIG, "  OpenGL version: " << glGetString(GL_VERSION));
+    AVG_TRACE(Logger::CONFIG, "  OpenGL vendor: " << glGetString(GL_VENDOR));
+    AVG_TRACE(Logger::CONFIG, "  OpenGL renderer: " << glGetString(GL_RENDERER));
     switch (m_YCbCrMode) {
-        case NONE:
-            AVG_TRACE(Logger::CONFIG, "YCbCr texture support not enabled.");
+        case OGL_NONE:
+            AVG_TRACE(Logger::CONFIG, "  YCbCr texture support not enabled.");
             break;
         case OGL_MESA:
-            AVG_TRACE(Logger::CONFIG, "Using Mesa YCbCr texture support.");
+            AVG_TRACE(Logger::CONFIG, "  Using Mesa YCbCr texture support.");
             break;
         case OGL_APPLE:
-            AVG_TRACE(Logger::CONFIG, "Using Apple YCbCr texture support.");
+            AVG_TRACE(Logger::CONFIG, "  Using Apple YCbCr texture support.");
             break;
         case OGL_SHADER:
-            AVG_TRACE(Logger::CONFIG, "Using fragment shader YCbCr texture support.");
+            AVG_TRACE(Logger::CONFIG, "  Using fragment shader YCbCr texture support.");
+            break;
+    }
+    switch(m_TextureMode) {
+        case GL_TEXTURE_2D:
+            AVG_TRACE(Logger::CONFIG, "  Using power of 2 textures.");
+            break;
+        case GL_TEXTURE_RECTANGLE_NV:
+            AVG_TRACE(Logger::CONFIG, 
+                    "  Using NVidia texture rectangle extension.");
             break;
     }
     if (m_MultiSampleSamples == 1) {
-        AVG_TRACE(Logger::CONFIG, "Not using multisampling.");
+        AVG_TRACE(Logger::CONFIG, "  Not using multisampling.");
     } else {
-        AVG_TRACE(Logger::CONFIG, "Using multisampling with " << m_MultiSampleSamples 
+        AVG_TRACE(Logger::CONFIG, "  Using multisampling with " << m_MultiSampleSamples 
                 << " samples");
     }
+    switch (getMemoryModeSupported()) {
+        case PBO:
+            AVG_TRACE(Logger::CONFIG, "  Using pixel buffer objects.");
+            break;
+        case OGL:
+            AVG_TRACE(Logger::CONFIG, "  Not using GL memory extensions.");
+            break;
+    }
+        AVG_TRACE(Logger::CONFIG,
+                "  Max. texture size is " << getMaxTexSize());
 }
 
-static ProfilingZone PrepareRenderProfilingZone("  Root node: prepareRender");
-static ProfilingZone RootRenderProfilingZone("  Root node: render");
+static ProfilingZone PrepareRenderProfilingZone("Root node: prepareRender");
+static ProfilingZone RootRenderProfilingZone("Root node: render");
 
 void SDLDisplayEngine::render(AVGNodePtr pRootNode, bool bRenderEverything)
 {
@@ -389,7 +428,7 @@ void SDLDisplayEngine::setClipRect()
     m_ClipRects.push_back(DRect(0, 0, m_Width, m_Height));
 }
 
-static ProfilingZone PushClipRectProfilingZone("      pushClipRect");
+static ProfilingZone PushClipRectProfilingZone("pushClipRect");
 
 bool SDLDisplayEngine::pushClipRect(const DRect& rc, bool bClip)
 {
@@ -408,7 +447,7 @@ bool SDLDisplayEngine::pushClipRect(const DRect& rc, bool bClip)
     return true;
 }
 
-static ProfilingZone PopClipRectProfilingZone("      popClipRect");
+static ProfilingZone PopClipRectProfilingZone("popClipRect");
 
 void SDLDisplayEngine::popClipRect()
 {
@@ -477,16 +516,7 @@ void SDLDisplayEngine::clip()
     }
 }
 
-void SDLDisplayEngine::setDirtyRect(const DRect& rc) 
-{
-    m_DirtyRect = rc;
-    
-    AVG_TRACE(Logger::BLTS, "Dirty rect: " << m_DirtyRect.tl.x << "x" << 
-            m_DirtyRect.tl.y << ", width: " << m_DirtyRect.Width() << 
-            ", height: " << m_DirtyRect.Height());
-}
-
-static ProfilingZone SwapBufferProfilingZone("  Render - swap buffers");
+static ProfilingZone SwapBufferProfilingZone("Render - swap buffers");
 
 void SDLDisplayEngine::swapBuffers()
 {
@@ -514,12 +544,10 @@ bool SDLDisplayEngine::supportsBpp(int bpp)
 
 bool SDLDisplayEngine::hasRGBOrdering()
 {
-    // So far, false is alot faster on linux NVIDIA proprietary drivers.
-    // Not sure for other drivers.
-    return m_bShouldUseRGBOrder;
+    return false;
 }
 
-DisplayEngine::YCbCrMode SDLDisplayEngine::getYCbCrMode()
+YCbCrMode SDLDisplayEngine::getYCbCrMode()
 {
     return m_YCbCrMode;
 }
@@ -571,7 +599,7 @@ int SDLDisplayEngine::getBPP()
     
 void SDLDisplayEngine::checkYCbCrSupport()
 {
-    m_YCbCrMode = NONE;
+    m_YCbCrMode = OGL_NONE;
     if (queryOGLExtension("GL_ARB_fragment_shader") &&
         queryOGLExtension("GL_ARB_texture_rectangle") &&
         (queryOGLExtension("GL_ARB_pixel_buffer_object") || 
@@ -582,23 +610,21 @@ void SDLDisplayEngine::checkYCbCrSupport()
         )
     {
         m_YCbCrMode = OGL_SHADER;
-        string sProgramInit = 
-#ifdef __APPLE__
-            "#define textureRect texture2DRect\n"
-            "#define samplerRect sampler2DRect\n"
-#endif
-            "uniform samplerRect YTexture;\n"
-            "uniform samplerRect CbTexture;\n"
-            "uniform samplerRect CrTexture;\n"
+        string sProgramInit =
+            "#extension GL_ARB_texture_rectangle : enable\n"
+            
+            "uniform sampler2DRect YTexture;\n"
+            "uniform sampler2DRect CbTexture;\n"
+            "uniform sampler2DRect CrTexture;\n"
             "\n";
         string sProgram =
             sProgramInit + 
             "void main(void)\n"
             "{\n"
             "  vec3 YCbCr;\n"
-            "  YCbCr.r = textureRect(YTexture, gl_TexCoord[0].st).a-0.0625;\n"
-            "  YCbCr.g = textureRect(CbTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
-            "  YCbCr.b = textureRect(CrTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
+            "  YCbCr.r = texture2DRect(YTexture, gl_TexCoord[0].st).a-0.0625;\n"
+            "  YCbCr.g = texture2DRect(CbTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
+            "  YCbCr.b = texture2DRect(CrTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
             "  vec3 RGB;"
             "  RGB = YCbCr*mat3(1.16,  0.0,   1.60,\n"
             "                   1.16, -0.39, -0.81,\n"
@@ -613,9 +639,9 @@ void SDLDisplayEngine::checkYCbCrSupport()
             "void main(void)\n"
             "{\n"
             "  vec3 YCbCr;\n"
-            "  YCbCr.r = textureRect(YTexture, gl_TexCoord[0].st).a;\n"
-            "  YCbCr.g = textureRect(CbTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
-            "  YCbCr.b = textureRect(CrTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
+            "  YCbCr.r = texture2DRect(YTexture, gl_TexCoord[0].st).a;\n"
+            "  YCbCr.g = texture2DRect(CbTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
+            "  YCbCr.b = texture2DRect(CrTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
             "  vec3 RGB;"
             "  RGB = YCbCr*mat3(1,  0.0  , 1.40,\n"
             "                   1, -0.34, -0.71,\n"
@@ -638,13 +664,17 @@ void SDLDisplayEngine::checkYCbCrSupport()
 
 bool SDLDisplayEngine::initVBlank(int rate) {
     
-    if (rate > 0) {
+    if (rate > 0 && m_DesiredVSyncMode != VSYNC_NONE) {
 #ifdef __APPLE__
         CGLContextObj Context = CGLGetCurrentContext();
         if (Context == 0) {
             AVG_TRACE(Logger::WARNING,
                     "Mac VBlank setup failed in CGLGetCurrentContext().");
         }
+        if (rate != 1) {
+            AVG_TRACE(Logger::WARNING,
+                    "VBlank rate set to " << rate << " but Mac OS X only supports 1. Assuming 1.");
+        }
         const long l = 1;
         CGLError err = CGLSetParameter(Context, kCGLCPSwapInterval, &l);
         m_VBMethod = VB_APPLE;
@@ -663,7 +693,8 @@ bool SDLDisplayEngine::initVBlank(int rate) {
         } else {
             string sVendor = (const char *)(glGetString(GL_VENDOR));
             if (sVendor.find("VIA Technology") != string::npos || 
-                    !queryGLXExtension("GLX_SGI_video_sync"))
+                    !queryGLXExtension("GLX_SGI_video_sync") ||
+                    m_DesiredVSyncMode == VSYNC_DRI)
             {
                 m_dri_fd = open("/dev/dri/card0", O_RDWR);
                 if (m_dri_fd < 0)
@@ -687,16 +718,16 @@ bool SDLDisplayEngine::initVBlank(int rate) {
     switch(m_VBMethod) {
         case VB_SGI:
             AVG_TRACE(Logger::CONFIG, 
-                    "Using SGI OpenGL extension for vertical blank support.");
+                    "  Using SGI OpenGL extension for vertical blank support.");
             break;
         case VB_DRI:
-            AVG_TRACE(Logger::CONFIG, "Using DRI vertical blank support.");
+            AVG_TRACE(Logger::CONFIG, "  Using DRI vertical blank support.");
             break;
         case VB_APPLE:
-            AVG_TRACE(Logger::CONFIG, "Using Apple GL vertical blank support.");
+            AVG_TRACE(Logger::CONFIG, "  Using Apple GL vertical blank support.");
             break;
         case VB_NONE:
-            AVG_TRACE(Logger::CONFIG, "Vertical blank support disabled.");
+            AVG_TRACE(Logger::CONFIG, "  Vertical blank support disabled.");
             break;
     }
     return m_VBMethod != VB_NONE;
@@ -819,24 +850,72 @@ void SDLDisplayEngine::initInput()
     initJoysticks();
 }
 
+const char * getEventTypeName(unsigned char Type) 
+{
+    switch(Type) {
+            case SDL_ACTIVEEVENT:
+                return "SDL_ACTIVEEVENT";
+            case SDL_KEYDOWN:
+                return "SDL_KEYDOWN";
+            case SDL_KEYUP:
+                return "SDL_KEYUP";
+            case SDL_MOUSEMOTION:
+                return "SDL_MOUSEMOTION";
+            case SDL_MOUSEBUTTONDOWN:
+                return "SDL_MOUSEBUTTONDOWN";
+            case SDL_MOUSEBUTTONUP:
+                return "SDL_MOUSEBUTTONUP";
+            case SDL_JOYAXISMOTION:
+                return "SDL_JOYAXISMOTION";
+            case SDL_JOYBUTTONDOWN:
+                return "SDL_JOYBUTTONDOWN";
+            case SDL_JOYBUTTONUP:
+                return "SDL_JOYBUTTONUP";
+            case SDL_VIDEORESIZE:
+                return "SDL_VIDEORESIZE";
+            case SDL_VIDEOEXPOSE:
+                return "SDL_VIDEOEXPOSE";
+            case SDL_QUIT:
+                return "SDL_QUIT";
+            case SDL_USEREVENT:
+                return "SDL_USEREVENT";
+            case SDL_SYSWMEVENT:
+                return "SDL_SYSWMEVENT";
+            default:
+                return "Unknown SDL event type";
+    }
+}
+
 vector<Event *> SDLDisplayEngine::pollEvents()
 {
     SDL_Event sdlEvent;
     vector<Event *> Events;
 
     while(SDL_PollEvent(&sdlEvent)){
-        switch(sdlEvent.type){
+        switch(sdlEvent.type) {
+            case SDL_ACTIVEEVENT:
+                if (sdlEvent.active.state & SDL_APPMOUSEFOCUS) {
+                    m_bMouseOverApp = sdlEvent.active.gain;
+                    if (!sdlEvent.active.gain) {
+                        Events.push_back(
+                                new MouseEvent (Event::CURSORMOTION, false,
+                                        false, false, IntPoint(-1, -1), 
+                                        MouseEvent::NO_BUTTON));
+                    }
+                }
             case SDL_MOUSEMOTION:
-                Events.push_back
-                        (createMouseMotionEvent(Event::MOUSEMOTION, sdlEvent));
+                if (m_bMouseOverApp) {
+                    Events.push_back
+                            (createMouseMotionEvent(Event::CURSORMOTION, sdlEvent));
+                }
                 break;
             case SDL_MOUSEBUTTONDOWN:
                 Events.push_back
-                        (createMouseButtonEvent(Event::MOUSEBUTTONDOWN, sdlEvent));
+                        (createMouseButtonEvent(Event::CURSORDOWN, sdlEvent));
                 break;
             case SDL_MOUSEBUTTONUP:
                 Events.push_back
-                        (createMouseButtonEvent(Event::MOUSEBUTTONUP, sdlEvent));
+                        (createMouseButtonEvent(Event::CURSORUP, sdlEvent));
                 break;
             case SDL_JOYAXISMOTION:
 //                Events.push_back(createAxisEvent(sdlEvent));
@@ -855,7 +934,7 @@ vector<Event *> SDLDisplayEngine::pollEvents()
                 break;
             case SDL_QUIT:
                 {
-                    Event * pEvent = new Event(Event::QUIT);
+                    Event * pEvent = new Event(Event::QUIT, Event::NONE);
                     Events.push_back(pEvent);
                 }
                 break;
@@ -878,7 +957,7 @@ Event * SDLDisplayEngine::createMouseMotionEvent
             SDLEvent.motion.state & SDL_BUTTON(1),
             SDLEvent.motion.state & SDL_BUTTON(3), 
             SDLEvent.motion.state & SDL_BUTTON(2),
-            x, y, 
+            IntPoint(x, y), 
             MouseEvent::NO_BUTTON);
     return pEvent;
 }
@@ -906,7 +985,7 @@ Event * SDLDisplayEngine::createMouseButtonEvent
             SDLEvent.button.button == SDL_BUTTON_LEFT,
             SDLEvent.button.button == SDL_BUTTON_MIDDLE, 
             SDLEvent.button.button == SDL_BUTTON_RIGHT,
-            x, y, Button);
+            IntPoint(x, y), Button);
     return pEvent; 
 }
 
@@ -1235,21 +1314,15 @@ int SDLDisplayEngine::getTextureMode()
     if (m_TextureMode == 0) {
         if (m_bShouldUsePOW2Textures) {
             m_TextureMode = GL_TEXTURE_2D;
-            AVG_TRACE(Logger::CONFIG, "Using power of 2 textures.");
         } else {
             if (queryOGLExtension("GL_NV_texture_rectangle")) {
                 m_TextureMode = GL_TEXTURE_RECTANGLE_NV;
-                AVG_TRACE(Logger::CONFIG, 
-                        "Using NVidia texture rectangle extension.");
             } else if (queryOGLExtension("GL_EXT_texture_rectangle") ||
                     queryOGLExtension("GL_ARB_texture_rectangle")) 
             {
                 m_TextureMode = GL_TEXTURE_RECTANGLE_ARB;
-                AVG_TRACE(Logger::CONFIG, 
-                        "Using portable texture rectangle extension.");
             } else {
                 m_TextureMode = GL_TEXTURE_2D;
-                AVG_TRACE(Logger::CONFIG, "Using power of 2 textures.");
             }
         }
     }
@@ -1260,8 +1333,6 @@ int SDLDisplayEngine::getMaxTexSize()
 {
     if (m_MaxTexSize == 0) {
         glGetIntegerv(GL_MAX_TEXTURE_SIZE, &m_MaxTexSize);
-        AVG_TRACE(Logger::CONFIG,
-                "Max. texture size is " << m_MaxTexSize);
     }
     return m_MaxTexSize;
 }
@@ -1283,9 +1354,9 @@ int SDLDisplayEngine::getOGLDestMode(PixelFormat pf)
             return GL_RGBA;    
         case YCbCr422:
             switch (getYCbCrMode()) {
-                case DisplayEngine::OGL_MESA:
+                case OGL_MESA:
                     return GL_YCBCR_MESA;    
-                case DisplayEngine::OGL_APPLE:
+                case OGL_APPLE:
                     return GL_RGBA;
                 default:
                     AVG_TRACE(Logger::ERROR, 
@@ -1316,9 +1387,9 @@ int SDLDisplayEngine::getOGLSrcMode(PixelFormat pf)
             return GL_RGBA;
         case YCbCr422:
             switch (getYCbCrMode()) {
-                case DisplayEngine::OGL_MESA:
+                case OGL_MESA:
                     return GL_YCBCR_MESA;    
-                case DisplayEngine::OGL_APPLE:
+                case OGL_APPLE:
                     return GL_YCBCR_422_APPLE;
                 default:
                     AVG_TRACE(Logger::ERROR, "SDLDisplayEngine: YCbCr422 not supported.");
@@ -1335,15 +1406,11 @@ int SDLDisplayEngine::getOGLPixelType(PixelFormat pf)
 {
     switch (pf) {
         case YCbCr422:
-            if (getYCbCrMode() == DisplayEngine::OGL_MESA) {
-                return GL_UNSIGNED_SHORT_8_8_REV_MESA;
-            } else {
 #ifdef __i386__
-                return GL_UNSIGNED_SHORT_8_8_REV_APPLE;
+            return GL_UNSIGNED_SHORT_8_8_REV_APPLE;
 #else
-                return GL_UNSIGNED_SHORT_8_8_APPLE;
+            return GL_UNSIGNED_SHORT_8_8_APPLE;
 #endif 
-            }
         case B8G8R8X8:
         case B8G8R8A8:
             return GL_UNSIGNED_INT_8_8_8_8_REV;
@@ -1360,10 +1427,8 @@ OGLMemoryMode SDLDisplayEngine::getMemoryModeSupported()
             m_bShouldUsePixelBuffers)
         {
             m_MemoryMode = PBO;
-            AVG_TRACE(Logger::CONFIG, "Using pixel buffer objects.");
         } else {
             m_MemoryMode = OGL;
-            AVG_TRACE(Logger::CONFIG, "Not using GL memory extensions.");
         }
         m_bCheckedMemoryMode = true;
     }
@@ -1373,7 +1438,8 @@ OGLMemoryMode SDLDisplayEngine::getMemoryModeSupported()
 
 
 void SDLDisplayEngine::setOGLOptions(bool bUsePOW2Textures, YCbCrMode DesiredYCbCrMode, 
-        bool bUseRGBOrder, bool bUsePixelBuffers, int MultiSampleSamples)
+        bool bUsePixelBuffers, int MultiSampleSamples, 
+        VSyncMode DesiredVSyncMode)
 {
     if (m_pScreen) {
         AVG_TRACE(Logger::ERROR, 
@@ -1382,9 +1448,9 @@ void SDLDisplayEngine::setOGLOptions(bool bUsePOW2Textures, YCbCrMode DesiredYCb
     }
     m_bShouldUsePOW2Textures = bUsePOW2Textures;
     m_DesiredYCbCrMode = DesiredYCbCrMode;
-    m_bShouldUseRGBOrder = bUseRGBOrder;
     m_bShouldUsePixelBuffers = bUsePixelBuffers;
     m_MultiSampleSamples = MultiSampleSamples;
+    m_DesiredVSyncMode = DesiredVSyncMode;
 }
 
 }
