@@ -92,8 +92,9 @@ class SDLDisplayEngine: public DisplayEngine, public IEventSource
         int getOGLPixelType(PixelFormat pf);
         OGLMemoryMode getMemoryModeSupported();
 
-        void setOGLOptions(bool bUsePOW2Textures, YCbCrMode ycbcrMode, bool bUseRGBOrder,
-                bool bUsePixelBuffers, int MultiSampleSamples);
+        void setOGLOptions(bool bUsePOW2Textures, YCbCrMode DesiredYcbcrMode, 
+                bool bUsePixelBuffers, int MultiSampleSamples, 
+                VSyncMode DesiredVSyncMode);
 
     private:
         void initSDL(int width, int height, bool isFullscreen, int bpp);
@@ -101,7 +102,6 @@ class SDLDisplayEngine: public DisplayEngine, public IEventSource
         void initTranslationTable();
         void initJoysticks();
         void logConfig(); 
-        void setDirtyRect(const DRect& rc);
         virtual void swapBuffers();
         void clip();
         void setClipPlane(double Eqn[4], int WhichPlane);
@@ -121,7 +121,6 @@ class SDLDisplayEngine: public DisplayEngine, public IEventSource
         int m_WindowWidth;
         int m_WindowHeight;
         std::vector<DRect> m_ClipRects;
-        DRect m_DirtyRect;
         bool m_bEnableCrop;
 
         SDL_Surface * m_pScreen;
@@ -144,6 +143,7 @@ class SDLDisplayEngine: public DisplayEngine, public IEventSource
         static void calcRefreshRate();
         static double s_RefreshRate;
 
+        bool m_bMouseOverApp;
         static std::vector<long> KeyCodeTranslationTable;
 
         // Texture config.
@@ -152,9 +152,9 @@ class SDLDisplayEngine: public DisplayEngine, public IEventSource
 
         bool m_bShouldUsePOW2Textures;
         YCbCrMode m_DesiredYCbCrMode;
-        bool m_bShouldUseRGBOrder;
         bool m_bShouldUsePixelBuffers;
         int m_MultiSampleSamples;
+        VSyncMode m_DesiredVSyncMode;
 
         bool m_bCheckedMemoryMode;
         OGLMemoryMode m_MemoryMode;
