@@ -23,6 +23,10 @@
 #include "Player.h"
 #include "MouseEvent.h"
 
+#include "../base/ObjectCounter.h"
+
+#include <iostream>
+
 using namespace std;
 
 namespace avg {
@@ -41,16 +45,26 @@ int TestHelper::getNumDifferentPixels(Bitmap* pBmp1, Bitmap* pBmp2)
     return pBmp1->getNumDifferentPixels(*pBmp2);
 }
 
+void TestHelper::useFakeCamera(bool bFake)
+{
+    m_pPlayer->useFakeCamera(bFake);
+}
+
 void TestHelper::fakeMouseEvent(Event::Type eventType,
         bool leftButtonState, bool middleButtonState, 
         bool rightButtonState,
         int xPosition, int yPosition, int button)
 {
     MouseEvent * pEvent = new MouseEvent(eventType, leftButtonState, 
-            middleButtonState, rightButtonState, xPosition, yPosition, button);
+            middleButtonState, rightButtonState, IntPoint(xPosition, yPosition), button);
     m_Events.push_back(pEvent);
 }
 
+void TestHelper::dumpObjects()
+{
+    cerr << ObjectCounter::get()->dump();
+}
+
 // From IEventSource
 std::vector<Event *> TestHelper::pollEvents()
 {
