@@ -22,9 +22,14 @@
 #ifndef _Video_H_
 #define _Video_H_
 
+#include <Python.h> // Python docs say this should be included before 
+                    // any standard headers (!)
+
 #include "Node.h"
 #include "VideoBase.h"
-#include "../graphics/Rect.h"
+
+#include "../base/Point.h"
+#include "../base/IFrameListener.h"
 
 #include <string>
 
@@ -32,10 +37,9 @@ namespace avg {
 
 class IVideoDecoder;
 
-class Video : public VideoBase
+class Video : public VideoBase, IFrameListener
 {
     public:
-        Video ();
         Video (const xmlNodePtr xmlNode, Player * pPlayer);
         virtual ~Video ();
         
@@ -47,10 +51,17 @@ class Video : public VideoBase
         
         int getNumFrames() const;
         int getCurFrame() const;
-        void seekToFrame(int num);
+        void seekToFrame(int FrameNum);
         bool getLoop() const;
+        bool isThreaded() const;
+        void setEOFCallback(PyObject * pEOFCallback);
 
-        virtual std::string getTypeStr ();
+        virtual std::string getTypeStr();
+
+        virtual void onFrameEnd();
+
+    protected:
+        virtual void changeVideoState(VideoState NewVideoState);
 
     private:
         void initVideoSupport();
@@ -58,19 +69,29 @@ class Video : public VideoBase
         bool renderToSurface(ISurface * pSurface);
         bool canRenderToBackbuffer(int BitsPerPixel);
         void seek(int DestFrame);
-        void advancePlayback();
+        void onEOF();
        
-        virtual void open(int* pWidth, int* pHeight);
+        virtual void open(YCbCrMode ycbcrMode);
         virtual void close();
-        virtual PixelFormat getDesiredPixelFormat();
+        virtual PixelFormat getPixelFormat();
+        virtual IntPoint getSize();
         virtual double getFPS();
+        virtual long long getCurTime();
 
         std::string m_href;
         std::string m_Filename;
         bool m_bLoop;
+        bool m_bThreaded;
+        double m_FPS;
+        bool m_bEOFPending;
+        PyObject * m_pEOFCallback;
+        int m_FramesTooLate;
+        int m_FramesPlayed;
 
         int m_CurFrame;
-        bool m_bEOF;
+        long long m_StartTime;
+        long long m_PauseTime;
+        long long m_PauseStartTime;
 
         IVideoDecoder * m_pDecoder;
 
