@@ -23,7 +23,8 @@
 #define _VideoBase_H_
 
 #include "RasterNode.h"
-#include "../graphics/Rect.h"
+
+#include "../base/Rect.h"
 
 #include <string>
 
@@ -36,6 +37,7 @@ class VideoBase : public RasterNode
     public:
         virtual ~VideoBase ();
         void setDisplayEngine(DisplayEngine * pEngine);
+        void disconnect();
         
         void play();
         void stop();
@@ -48,34 +50,29 @@ class VideoBase : public RasterNode
         virtual std::string dump (int indent = 0);
         
     protected:        
-        VideoBase ();
         VideoBase (const xmlNodePtr xmlNode, Player * pPlayer);
         virtual DPoint getPreferredMediaSize();
         typedef enum VideoState {Unloaded, Paused, Playing};
         virtual VideoState getVideoState() const;
         void setFrameAvailable(bool bAvailable);
-        void changeVideoState(VideoState NewVideoState);
+        virtual void changeVideoState(VideoState NewVideoState);
         int getMediaWidth();
         int getMediaHeight();
-        DisplayEngine::YCbCrMode getYCbCrMode();
    
     private:
         void renderToBackbuffer();
         void open();
 
         virtual bool renderToSurface(ISurface * pSurface) = 0;
-        virtual bool canRenderToBackbuffer(int BitsPerPixel) = 0;
-        virtual void open(int* pWidth, int* pHeight) = 0;
+        virtual void open(YCbCrMode ycbcrMode) = 0;
         virtual void close() = 0;
-        virtual PixelFormat getDesiredPixelFormat() = 0;
+        virtual PixelFormat getPixelFormat() = 0;
+        virtual IntPoint getSize() = 0;
 
         VideoState m_VideoState;
-       
-        int m_Width;
-        int m_Height;
-        DisplayEngine::YCbCrMode m_YCbCrMode;
 
         bool m_bFrameAvailable;
+        bool m_bFirstFrameDecoded;
 };
 
 }
