@@ -41,33 +41,15 @@ using namespace std;
 
 namespace avg {
 
-Words::Words ()
-    : m_FontName("arial"),
-      m_Text(""),
-      m_ColorName("FFFFFF"),
-      m_Size(15),
-      m_ParaWidth(-1),
-      m_Indent(0),
-      m_LineSpacing(-1),
-      m_Alignment(PANGO_ALIGN_LEFT),
-      m_Weight(PANGO_WEIGHT_NORMAL),
-      m_bItalic(false),
-      m_Stretch(PANGO_STRETCH_NORMAL),
-      m_bSmallCaps(false),
-      m_pSurface(0),
-      m_StringExtents(0,0),
-      m_pContext(0),
-      m_pFontDescription(0),
-      m_bFontChanged(true),
-      m_bDrawNeeded(true)
-{
-}
+std::set<std::string> Words::s_sFontsNotFound;
 
 Words::Words (const xmlNodePtr xmlNode, Player * pPlayer)
     : RasterNode(xmlNode, pPlayer), 
-      m_pSurface(0), 
+      m_StringExtents(0,0),
       m_pContext(0), 
-      m_pFontDescription(0)
+      m_pFontDescription(0),
+      m_bFontChanged(true),
+      m_bDrawNeeded(true)
 {
     m_FontName = getDefaultedStringAttr (xmlNode, "font", "arial");
     m_Text = getDefaultedStringAttr (xmlNode, "text", "");
@@ -85,10 +67,9 @@ Words::Words (const xmlNodePtr xmlNode, Player * pPlayer)
 
 Words::~Words ()
 {
-    if (m_pSurface) {
-        delete m_pSurface;
-        m_pSurface = 0;
+    if (m_pContext) {
         g_object_unref(m_pContext);
+        m_pContext = 0;
         pango_font_description_free(m_pFontDescription);
     }
 }
@@ -114,7 +95,6 @@ text_subst_func (FcPattern *pattern, gpointer data)
 void Words::setDisplayEngine (DisplayEngine * pEngine)
 {
     m_Color = colorStringToColor(m_ColorName);
-    m_pSurface = pEngine->createSurface();
 
     pango_ft2_get_context(72, 72);
     
@@ -129,7 +109,7 @@ void Words::setDisplayEngine (DisplayEngine * pEngine)
             pango_language_from_string ("en_US"));
     pango_context_set_base_dir(m_pContext, PANGO_DIRECTION_LTR);
     m_pFontDescription = pango_font_description_new();
-    
+
     m_bFontChanged = true;
     m_bDrawNeeded = true;
     RasterNode::setDisplayEngine(pEngine);
@@ -137,9 +117,7 @@ void Words::setDisplayEngine (DisplayEngine * pEngine)
 
 void Words::disconnect()
 {
-    if (m_pSurface) {
-        delete m_pSurface;
-        m_pSurface = 0;
+    if (m_pContext) {
         g_object_unref(m_pContext);
         m_pContext = 0;
         pango_font_description_free(m_pFontDescription);
@@ -155,7 +133,6 @@ string Words::getTypeStr ()
 
 void Words::setAlignment(const string& sAlign)
 {
-    invalidate();
     if (sAlign == "left") {
         m_Alignment = PANGO_ALIGN_LEFT;
     } else if (sAlign == "center") {
@@ -167,12 +144,10 @@ void Words::setAlignment(const string& sAlign)
     }
 
     m_bDrawNeeded = true;
-    invalidate();
 }
 
 void Words::setWeight(const string& sWeight)
 {
-    invalidate();
     if (sWeight == "ultralight") {
         m_Weight = PANGO_WEIGHT_ULTRALIGHT;
     } else if (sWeight == "light") {
@@ -192,12 +167,10 @@ void Words::setWeight(const string& sWeight)
     }
     m_bFontChanged = true;
     m_bDrawNeeded = true;
-    invalidate();
 }
 
 void Words::setStretch(const string& sStretch)
 {
-    invalidate();
     if (sStretch == "ultracondensed") {
         m_Stretch = PANGO_STRETCH_ULTRA_CONDENSED;
     } else if (sStretch == "extracondensed") {
@@ -223,7 +196,6 @@ void Words::setStretch(const string& sStretch)
     }
     m_bFontChanged = true;
     m_bDrawNeeded = true;
-    invalidate();
 }
 
 string Words::getAlignment() const
@@ -293,7 +265,7 @@ bool equalIgnoreCase(const string& s1, const string& s2) {
     return sUpper1 == sUpper2;
 }
 
-static ProfilingZone DrawStringProfilingZone("  Words::drawString");
+static ProfilingZone DrawStringProfilingZone("Words::drawString");
 
 void Words::drawString()
 {
@@ -305,6 +277,7 @@ void Words::drawString()
         m_StringExtents = DPoint(0,0);
     } else {
         if (m_bFontChanged) {
+            AVG_TRACE(Logger::MEMORY, "Opening font " << m_FontName);
             pango_font_description_set_family(m_pFontDescription,
                     g_strdup(m_FontName.c_str()));
             pango_font_description_set_style(m_pFontDescription,
@@ -328,8 +301,11 @@ void Words::drawString()
             PangoFontDescription * pUsedDescription = pango_font_describe(pUsedFont);
             string sUsedName = pango_font_description_get_family(pUsedDescription);
             if (!equalIgnoreCase(sUsedName, m_FontName)) {
-                AVG_TRACE(Logger::WARNING, "Could not find font face " << m_FontName <<
-                        ". Using " << sUsedName << " instead.");
+                if (s_sFontsNotFound.find(m_FontName) == s_sFontsNotFound.end()) {
+                    AVG_TRACE(Logger::WARNING, "Could not find font face " << m_FontName <<
+                            ". Using " << sUsedName << " instead.");
+                    s_sFontsNotFound.insert(m_FontName);
+                }
             } else {
                 if (m_Weight != pango_font_description_get_weight(m_pFontDescription)) {
                     AVG_TRACE(Logger::WARNING, "Font face " << m_FontName <<
@@ -381,9 +357,9 @@ void Words::drawString()
         if (m_StringExtents.y == 0) {
             m_StringExtents.y = 1;
         }
-        m_pSurface->create(IntPoint(m_StringExtents), I8, false);
+        getSurface()->create(IntPoint(m_StringExtents), I8, false);
 
-        BitmapPtr pBmp = m_pSurface->lockBmp();
+        BitmapPtr pBmp = getSurface()->lockBmp();
         FilterFill<unsigned char>(0).applyInPlace(pBmp);
         FT_Bitmap bitmap;
         bitmap.rows = (int)m_StringExtents.y;
@@ -397,13 +373,15 @@ void Words::drawString()
         // Use 1 as x-position here to make sure italic text is never cut off.
         pango_ft2_render_layout(&bitmap, layout, 1, 0);
 
-        getEngine()->surfaceChanged(m_pSurface);
+        getEngine()->surfaceChanged(getSurface());
         if (m_LineSpacing == -1) {
             m_LineSpacing = pango_layout_get_spacing(layout)/PANGO_SCALE;
         }
         g_object_unref(layout);
+        
     }
     m_bDrawNeeded = false;
+
     setViewport(-32767, -32767, -32767, -32767);
 }
 
@@ -415,7 +393,7 @@ void Words::prepareRender(int time, const DRect& parent)
     Node::prepareRender(time, parent);
 }
 
-static ProfilingZone RenderProfilingZone("    Words::render");
+static ProfilingZone RenderProfilingZone("Words::render");
 
 void Words::render(const DRect& Rect)
 {
@@ -423,8 +401,8 @@ void Words::render(const DRect& Rect)
     if (m_Text.length() != 0 && getEffectiveOpacity() > 0.001) {
         DRect TextPos = getAbsViewport();
         TextPos.tl.x--;   // Compensate for italic hack in call to pango_ft2_render_layout
-        TextPos.br.x--; 
-        getEngine()->blta8(m_pSurface, &TextPos,
+        TextPos.br.x--;
+        getEngine()->blta8(getSurface(), &TextPos,
                 getEffectiveOpacity(), m_Color, getAngle(),
                 getPivot(), getBlendMode());
     }
