@@ -21,11 +21,11 @@
 
 const char * g_pAVGDTD = 
 "<!ENTITY % graphicsElementEvents\n"
-"  \"onmousedown CDATA #IMPLIED\n"
-"    onmouseup CDATA #IMPLIED\n"
-"    onmouseover CDATA #IMPLIED\n"
-"    onmousemove CDATA #IMPLIED\n"
-"    onmouseout CDATA #IMPLIED\" >\n"
+"  \"oncursordown CDATA #IMPLIED\n"
+"    oncursorup CDATA #IMPLIED\n"
+"    oncursormove CDATA #IMPLIED\n"
+"    oncursorover CDATA #IMPLIED\n"
+"    oncursorout CDATA #IMPLIED\" >\n"
 
 "<!ENTITY % nodeAttrs\n"
 "   \"id ID #IMPLIED\n"
@@ -64,7 +64,7 @@ const char * g_pAVGDTD =
 "<!ELEMENT image EMPTY>\n"
 "<!ATTLIST image\n"
 "   %rasterAttrs;\n"
-"   href CDATA #REQUIRED\n"
+"   href CDATA #IMPLIED\n"
 "   hue CDATA #IMPLIED\n"
 "   saturation CDATA #IMPLIED >\n"
 
@@ -72,13 +72,14 @@ const char * g_pAVGDTD =
 "<!ATTLIST video\n"
 "   %rasterAttrs;\n"
 "   href CDATA #REQUIRED\n"
-"   loop CDATA #IMPLIED>\n"
+"   loop CDATA #IMPLIED\n"
+"   fps CDATA #IMPLIED\n"
+"   threaded CDATA #IMPLIED>\n"
 
 "<!ELEMENT camera EMPTY>\n"
 "<!ATTLIST camera\n"
 "   %rasterAttrs;\n"
 "   device CDATA #REQUIRED\n"
-"   mode CDATA #IMPLIED\n"
 "   framerate CDATA #IMPLIED\n"
 "   brightness CDATA #IMPLIED\n"
 "   exposure CDATA #IMPLIED\n"
@@ -86,7 +87,12 @@ const char * g_pAVGDTD =
 "   saturation CDATA #IMPLIED\n"
 "   gamma CDATA #IMPLIED\n"
 "   shutter CDATA #IMPLIED\n"
-"   whitebalance CDATA #IMPLIED>\n"
+"   whitebalance CDATA #IMPLIED\n"
+"   source CDATA #IMPLIED\n"
+"   pixelformat CDATA #IMPLIED\n"
+"   capturewidth CDATA #IMPLIED\n"
+"   captureheight CDATA #IMPLIED\n"
+"   channel CDATA #IMPLIED>\n"
 
 "<!ELEMENT words (#PCDATA|span|b|big|i|s|sub|sup|small|tt|u)*>\n"
 "<!ATTLIST words\n"
